<?php
class ResponsiveGoogleMaps_MarkAdCoreClasses_Help
{
	protected $hook;
	protected $help_tab_title;
	protected $help_content;
	protected $sidebar_content;
	protected $screen;
	protected $placement;
	protected $hook_filter = false;
	
	public static function AddMenuTab($hook, $help_tab_title = NULL, $help_content = NULL){
		$class = new ResponsiveGoogleMaps_MarkAdCoreClasses_Help($hook, $help_tab_title, $help_content);
	}
	
	public static function HelpSidebar($hook, $sidebar_content = NULL, $placement = NULL){
		$class = new ResponsiveGoogleMaps_MarkAdCoreClasses_Help($hook, NULL, NULL, $sidebar_content, $placement);
	}
	
	protected function __construct($hook, $help_tab_title = NULL, $help_content = NULL, $sidebar_content = NULL, $placement = NULL){	
		// Set Parameters
		$this->hook = $hook;
	 	$this->help_tab_title = $help_tab_title;
		$this->placement = $placement;
		
		// set help content
		if($help_content === strip_tags($help_content))	{
			// not html, might be a file
			$this->help_content = $this->GetHelpFileHTML($help_content);
		}
		else {
			$this->help_content = $help_content;
		}
		
		// set sidebat content
		if($sidebar_content === strip_tags($sidebar_content)){
			// not html, might be a file
			$this->sidebar_content = $this->GetHelpFileHTML($sidebar_content);
		}
		else {
			$this->sidebar_content = $sidebar_content;
		}
		
		// Add Action(s)
		if(is_array($hook)){
			foreach($hook as $single_hook)
				$this->ParseHook($single_hook);
		}
		else
			$this->ParseHook($hook);
	}
	
	public function ParseHook($hook){
		if(in_array('?', str_split($hook))){
			$hook = explode('?', $hook);
			$this->hook_filter = $hook[1];
			add_action($hook[0], array($this, 'ActionHookFunction'));
		}
		else
			add_action($hook, array($this, 'ActionHookFunction'));
	}
	
	public function ActionHookFunction(){
		$this->screen = get_current_screen();
		if($this->hook_filter){
			$filter = explode('=', $this->hook_filter);
			if($this->screen->$filter[0] != $filter[1])
				return;
		}
		
		if($this->help_content !== NULL && $this->help_tab_title !== NULL)
			$this->screen->add_help_tab(array('id' => uniqid(), 'title' => $this->help_tab_title, 'content' => $this->help_content));
		
		if($this->sidebar_content !== NULL && $this->sidebar_content !== true)
			add_filter( 'contextual_help', array($this, 'AddHelpSidebarContent'), 5 );
	}
	
	public function AddHelpSidebarContent(){
		$content = '';
		$current_sidebar = $this->screen->get_help_sidebar();
		switch($this->placement){
			case 'prepend':
				$content = $current_sidebar.$this->sidebar_content;
			break;
			case 'replace':
				$content = $this->sidebar_content;
			break;
			case 'append':
				$content = $current_sidebar.$this->sidebar_content;
			break;
			default:
				//append
				$content = $current_sidebar.$this->sidebar_content;
			break;
		}
		$this->screen->set_help_sidebar($content);
	}
	
	protected function GetHelpFileHTML($help_file_name){	
		// Check if File Exists
		if(file_exists($help_file_name)){
			$ext = pathinfo($help_file_name, PATHINFO_EXTENSION);
			if($ext == 'php')
				return include($help_file_name);
			else
				// It's only a help file, supress errors if we have them
				return @file_get_contents($help_file_name);
		}
		return;
	}
}
?>