<?php
class ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox
{
	protected $lang = '';
	protected $settings;
	protected $settings_ref;
	protected $saved_values;
	protected $form_fields;
	
	public function __construct($settings = NULL){
		//return if we aren't in admin view
		if(!is_admin())
			return;
		
		if($settings !== NULL)	{
			$this->settings_ref = $settings;
			//instanciate this class right away to enqueue proper scripts and styles
			$this->form_fields = new ResponsiveGoogleMaps_MarkAdCoreClasses_FormFields();
			add_action('add_meta_boxes', array($this, 'add_metabox'));
			add_action('save_post', array($this, 'save_metabox'));
			add_action('admin_enqueue_scripts', array($this, 'add_scripts_and_styles'));
		}
	}

	public function load_settings($settings)	{
		//delay the include of file unitl $post is available
		if(is_string($settings) && file_exists($settings))
			$this->settings = include $settings;
		else if(is_array($settings))
			$this->settings = $settings;
	}
	
	public function add_metabox(){
		// Only load the settings if we need them on page load
		$this->load_settings($this->settings_ref);
		if(is_array($this->settings)){
			if(array_key_exists('text_domain', $this->settings)){
				$this->lang = $this->settings['text_domain'];
				$this->form_fields->Load_lang($this->lang); // text_domain
			}
	
			add_meta_box( 
				$this->settings['metabox_id'],
				__($this->settings['metabox_title'], $this->lang),
				array($this, 'render_metabox_content'),
				$this->settings['post_type'],
				$this->settings['context'],
				$this->settings['priority']
			);
		}
	}
	
	public function render_metabox_content($post){
		$output = '';
		$has_groups = false;
		$settings = $this->settings;
		wp_nonce_field($settings['metabox_id'], '_noncename_'.$settings['metabox_id']);
		
		$this->saved_values = get_post_meta($post->ID, $settings['metabox_id'], true);

		if(!$this->saved_values && array_key_exists('defaults', $settings))
			$this->saved_values = $settings['defaults'];
				
		$this->post = $post;
		
		$fields = $settings['meta_fields'];
		if(is_array($fields)){
			if(array_key_exists('before_metabox', $settings))
				$output .= $settings['before_metabox'];
			if(array_key_exists('groups', $settings)){
				$has_groups = true;
				$group_start = array();
				$group_end = array();
				$tab_html = '<ul class="meta-tabs">';
				foreach($settings['groups'] as $group){
					$id = uniqid();
					$group_start[$group['start']] = 'content-'.$id;
					$group_end[$group['end']] = $id;
					$tab_html .= '<li><a href="#'.$group_start[$group['start']].'" id="'.$id.'">'.__($group['label'], $this->lang).'</a></li>';
				}
				$tab_html .= '</ul>';
			}
			$counter = 0;
			foreach($fields as $field){
				if($has_groups){
					if(array_key_exists('before_group', $settings) && $counter == min(array_flip($group_start))){
						$output .= $settings['before_group'];
						$output .= $tab_html;
					}
					if(array_key_exists($counter, $group_start))
						$output .= '<div id="'.$group_start[$counter].'" class="meta-tab-content-container">';
				}
				
				if(is_array($field['id'])){
					$values = $this->saved_values;
					foreach($field['id'] as $id_part){
						if(array_key_exists($id_part, $values))
							$values = $values[$id_part];
						else {
							$values = '';
							break;	
						}
					}
					$passed_values = $values;
				}					
				elseif(is_string($field['id']) && array_key_exists($field['id'], (array)$this->saved_values))
					$passed_values = $this->saved_values[$field['id']];
				else
					$passed_values = NULL;
				//////////////////////
				$field_output = '';
				if(array_key_exists('before_field', $settings))
					$field_output .= $settings['before_field'];
				
				$field_output .= $this->form_fields->$field['type'](array($field, $settings['metabox_id'], $passed_values));
				if(array_key_exists('after_field', $settings))
					$field_output .= $settings['after_field'];
				
				$output .= apply_filters('rgm_metabox_'.$settings['metabox_id'].'_'.$field['id'], $field_output);
				///////////////////////	
				if($has_groups){
					if(array_key_exists($counter, $group_end))
						$output .= '</div>';
					if(array_key_exists('after_group', $settings) && $counter == max(array_flip($group_end)))
						$output .= $settings['after_group'];
				}
				$counter++;
			}
			if(array_key_exists('after_metabox', $settings))
				$output .= $settings['after_metabox'];
		}
		
		$output = apply_filters('rgm_metabox_'.$settings['metabox_id'], $output);
		echo $output;
	}
	
	public function save_metabox($post_id) 
	{
		// Only load the settings if we need them on page save
		$this->load_settings($this->settings_ref);
		$settings = $this->settings;
		
		if (!isset( $_POST['_noncename_'.$settings['metabox_id']]) || !wp_verify_nonce( $_POST['_noncename_'.$settings['metabox_id']], $settings['metabox_id']))
			return;
	
		
		if ($this->settings['post_type'] != $_POST['post_type']){
			return;
		} else if(!current_user_can('edit_page', $post_id)){
				return;
		} else {
			if (!current_user_can('edit_post', $post_id))
				return;
		}
		
		if(array_key_exists($settings['metabox_id'], $_POST)){
			$data_to_save = $_POST[$settings['metabox_id']];
			foreach($settings['meta_fields']	as $fields)	{
				// we have a checkbox set and no data to save. force a false
				if(array_key_exists('type', $fields) && $fields['type'] == 'checkbox' && !array_key_exists($fields['id'], $data_to_save)){
					$data_to_save[$fields['id']] = false;
				}
				if(is_array($fields['id'])){
					foreach($fields['id'] as $id){
						global $post;
						//value save callback call
						call_user_func_array($fields['save_callback'], array(&$data_to_save[$id], $post->ID));
					}
				}
				else if(array_key_exists($fields['id'], $data_to_save) && array_key_exists('save_callback', $fields)){
					global $post;
					//value save callback call
					call_user_func_array($fields['save_callback'], array(&$data_to_save[$fields['id']], $post->ID));
				}
			}
		}
		else
			$data_to_save = '';

		update_post_meta($post_id, $settings['metabox_id'], $data_to_save);
	}
	
	public function add_scripts_and_styles(){
		global $post;
		$settings = $this->settings;
		
		if(is_object($post) && $post->post_type == $settings['post_type']){
			$this->form_fields->add_scripts_and_styles();
			wp_enqueue_style('markad-metabox-admin-styles', plugins_url('assets/admin-styles.css', __FILE__ ));
			if(array_key_exists('stylesheet_url',$settings)){
				if($settings['stylesheet_url'] && is_string($settings['stylesheet_url']))
					wp_enqueue_style('markad-user-metabox-admin-styles'.uniqid(), $settings['stylesheet_url']);
			}
			if(array_key_exists('javascript_url',$settings)){
				if($settings['javascript_url'] && is_string($settings['javascript_url']))
					wp_enqueue_script('markad-user-metabox-admin-scripts'.uniqid(), $settings['javascript_url']);
				if($settings['javascript_url'] && is_array($settings['javascript_url'])){
					foreach($settings['javascript_url'] as $url){
						wp_enqueue_script('markad-user-metabox-admin-scripts'.uniqid(), $url);
					}
				}
			}
			if(array_key_exists('localize',$settings)){
				if($settings['localize'] && is_array($settings['localize'])){
					foreach($settings['localize'] as $name => $object){
						wp_localize_script('jquery', $name, $object);
					}
				}
			}
		}
	}
}
?>