<?php
class ResponsiveGoogleMaps_MarkAdCoreClasses_Options
{
	public $menu_hook;
	private $options = array(); //retieved from database
	private $settings = array(); //loaded in constructor
	private $option_page_title;
	private $plugin_file;
	private $group_name;
	private $section_names = array();
	protected $form_fields;
	protected $lang = '';
	protected static $_instances = array();
	protected static $uninstall_ran = false;
	
	public function __construct($file_path = false, $plugin_file = NULL, $option_array = NULL){
		//return if we aren't in admin view
		if(!is_admin())
			return;
		
		self::$_instances[] = $this;
		$this->settings = $option_array;
		$this->plugin_file = $plugin_file;

		if($file_path !== NULL)
			$this->settings = include $file_path;
		
		// 'group_'. added to provide no-conflict with similar settings
		$this->group_name = 'group_'.$this->settings['group'];
		$this->option_page_title = $this->settings['page_title'];
		
		// Instanciate this class right away to enqueue proper scripts and styles
		$this->form_fields = new ResponsiveGoogleMaps_MarkAdCoreClasses_FormFields();
		if(array_key_exists('text_domain', $this->settings)){
			$this->lang = $this->settings['text_domain'];
			$this->form_fields->Load_lang($this->lang); // text_domain
		}
		
		if($plugin_file){
			register_activation_hook($this->plugin_file, array($this, 'install'));
			// For future use
			//register_deactivation_hook($this->plugin_file, array($this, 'deactivate'));
			
			/*
			Cannot register more than one uninstall function
			maybe this will change in the future
			*/
			//register_uninstall_hook($this->plugin_file, array(__CLASS__, 'uninstall'));
		}
		else
			add_action('admin_init', array($this, 'install'));
		
		add_action('admin_init', array($this, 'register_settings'));
		add_action('admin_menu', array($this, 'hook_menu'));
		add_action('admin_enqueue_scripts', array($this, 'add_scripts_and_styles'));
	}
	
	public function install(){
		$settings = $this->settings;
		
		foreach($settings['sections'] as $section){
			$section_name = $section['name'];
			
			$defaults = array();
			foreach($section['settings_fields'] as $field){
				if(array_key_exists('default',$field))
					$defaults[$field['id']] = $field['default'];
				else if(array_key_exists('fields',$field)){
					foreach($field['fields'] as $field_array){
						if($field_array['default'])
							$defaults[$field['id']] = array($field_array['value']);
					}
				}
			}
			if(false == ($current_options = get_option($section_name)))
				add_option($section_name, $defaults);
			else {
				$new_options = array_merge($current_options, array_diff_key($defaults, $current_options));
				update_option($section_name, $new_options);
			}
			register_setting( $this->group_name, $section_name );
		}
	}
	
	public function deactivate(){
		// for furure use
	}
	
	public static function uninstall(){
		if(!self::$uninstall_ran){
			foreach(self::$_instances as $instance){
				$settings = $instance->settings;
				
				foreach($settings['sections'] as $section){
					$section_name = $section['name'];
					delete_option($section_name);
					unregister_setting( $instance->group_name, $section_name );
				}
			}
			self::$uninstall_ran = true;
		}
	}
	
	public function register_settings(){
		$settings = $this->settings;
		
		foreach($settings['sections'] as $section){
			$section_name = $section['name'];
			$section_title = NULL;
			if(array_key_exists('title',$section))
				$section_title = $section['title'];
			
			$this->section_names[] = $section_name;
			// 'section_id_'. added to provide no-conflict with section name
			$section_id = 'section_id_'.$section_name;
			
			//load the options array for use in add_settings_field callback
			$this->options[$section_name] = get_option($section_name);

			register_setting( $this->group_name, $section_name );
			
			add_settings_section(
				$section_id,
				__($section_title, $this->lang),
				NULL,
				$section_name
			);	

			$fields = $section['settings_fields'];
			if(is_array($fields)){
				foreach($fields as $field){
					if(array_key_exists($field['id'], (array)$this->options[$section_name]))
						$passed_values = $this->options[$section_name][$field['id']];
					else
						$passed_values = NULL;
					add_settings_field(
						$field['id'], 
						__($field['title'], $this->lang), 
						//array($this->form_fields, $field['type']),
						array($this->form_fields, 'output_field'),
						$section_name,
						$section_id,
						array($field, $section_name, $passed_values)
					);
				}
			}
		}
	}
		
	//Output the settings page
	public function settings_page(){
		?>
		<div class="wrap">
			<div class="options_form">
            	<?php screen_icon(); ?>
				<h2><?php _e($this->option_page_title, $this->lang); ?></h2>
				<form method="post" action="options.php">
					<?php settings_fields( $this->group_name ); ?>
                  <?php
					foreach($this->section_names as $section){
						do_settings_sections($section);
					}
					?>
					<?php submit_button(); ?>
				</form>
			</div>
		</div>
		<?php 
	}
	
	public function hook_menu(){
		$settings = $this->settings;
		switch($settings['menu_type']){
			case 'top':
				$this->menu_hook = add_menu_page( __($settings['page_title'], $this->lang), __($settings['menu_title'], $this->lang), $settings['capability'], $settings['menu_slug'], array($this, 'settings_page'), $settings['icon_url'], (int)$settings['position'] );
			break;
			case 'sub':
				$this->menu_hook = add_submenu_page( $settings['parent_slug'], __($settings['page_title'], $this->lang), __($settings['menu_title'], $this->lang), $settings['capability'], $settings['menu_slug'], array($this, 'settings_page'));
			break;
		}
		if($this->menu_hook && array_key_exists('help_file', $settings) && $settings['help_file'])
			ResponsiveGoogleMaps_MarkAdCoreClasses_Help::AddMenuTab("load-$this->menu_hook", __($settings['help_title'], $this->lang), $settings['help_file']);
	}
	
	public function add_scripts_and_styles($hook = NULL){
		if($hook == $this->menu_hook){
			$this->form_fields->add_scripts_and_styles();
		}
	}
}
?>