<?php
class ResponsiveGoogleMaps_Plugin
{
	const LANG = 'responsive-google-maps';
	private static $instance = NULL;
	public static $global_settings_echoed = false;
	public static $settings = NULL;
	private $map_options_page;
	private $location_options_page;
	private $map_type_options_page;
	private $cluster_options_page;
	private $available_styles_array = array();
	private $external_style_ids = array();
	private $front_end_terms;
	private $front_end_domain = self::LANG;
	private $map_style_domain;
	
	private function __construct($settings = NULL){
		self::$settings = new ResponsiveGoogleMaps_MarkAdCoreClasses_Config($settings);
		$this->add_filters_and_actions();
	}
	
	public static function init($settings = NULL){
		if(self::$instance === NULL)
			self::$instance = new ResponsiveGoogleMaps_Plugin($settings);
		else
			return self::$instance;
	}
	
	public static function get_instance(){
		if(self::$instance !== NULL)
			return self::$instance;
		else
			return false;
	}
	
	protected function add_filters_and_actions()
	{
		// Filters
		add_filter('widget_text', 'do_shortcode');
		add_filter('plugin_row_meta', array($this, 'set_plugin_meta'), 10, 2);
		add_filter('single_template', array($this, 'set_single_template'));
		add_filter('archive_template', array($this, 'set_taxonomy_template'));
		add_filter('manage_'.self::$settings->get('map_slug').'_posts_columns', array($this, 'map_columns_head'));
		add_filter('manage_'.self::$settings->get('location_slug').'_posts_columns', array($this, 'location_columns_head'));
		add_filter('media_view_strings',  array($this, 'custom_media_string'), 10, 2);
		add_filter('pre_update_option_marker_cluster_settings', array($this, 'update_cluster_options'), 10, 2);
		
		// Actions
		add_action('wp_ajax_add_location_to_map', array($this, 'ajax_add_location_to_map'));
		add_action('wp_ajax_quick_add_location_to_map', array($this, 'ajax_quick_add_location_to_map'));
		add_action('wp_ajax_get_available_styles', array($this, 'ajax_get_available_styles'));
		add_action('wp_ajax_nopriv_get_available_styles', array($this, 'ajax_get_available_styles'));
		add_action('wp_ajax_get_locations_by_ids', array($this, 'ajax_get_locations_by_ids'));
		add_action('wp_ajax_nopriv_get_locations_by_ids', array($this, 'ajax_get_locations_by_ids'));
		add_action('init', array($this, 'register_scripts_and_styles'));
		add_action('init', array($this, 'register_taxonomies'));
		add_action('init', array($this, 'create_custom_post_types'));
		add_action('plugins_loaded', array($this, 'localize_plugin'));
		add_action('admin_menu', array($this, 'create_menu'), 10, 1);
		add_action('admin_enqueue_scripts', array($this, 'add_admin_scripts_and_styles'));
		add_action('wp_enqueue_scripts', array($this, 'check_for_shortcode'));
		add_action('manage_'.self::$settings->get('map_slug').'_posts_custom_column', array($this, 'map_columns_content'), 10, 2);
		add_action('manage_'.self::$settings->get('location_slug').'_posts_custom_column', array($this, 'location_columns_content'), 10, 2);
		
		// Custom map marker media manager
		add_action('wp_ajax_add_media_marker_tab_content', array($this, 'add_media_marker_tab_content'));
		add_action('admin_print_footer_scripts', array($this, 'print_marker_override_template'));
				
		// Activation hooks
		register_activation_hook(self::$settings->get('index_file'), array($this, 'plugin_activation'));
		register_deactivation_hook(self::$settings->get('index_file'), array($this, 'plugin_deactivation'));
		register_uninstall_hook(self::$settings->get('index_file'), array(__CLASS__, 'plugin_uninstall'));
		
		// Shortcodes
		add_shortcode( 'google_map', array($this, 'google_map'));
		add_shortcode( 'google_map_vc', array($this, 'google_map_vc'));
		
		// setup options pages and metabaoxes
		$this->add_options_pages();
		add_action('admin_init', array($this, 'add_metaboxes'));
		
		// Add Visual Composer Block
		add_action('admin_init', array($this, 'make_visual_composer_block'));
	}
	
	public function add_options_pages(){
		// We have to check is_admin here because actions run too late
		if(is_admin()){
			$this->map_options_page = new ResponsiveGoogleMaps_MarkAdCoreClasses_Options(dirname(self::$settings->get('index_file')).'/options/map-settings.php', self::$settings->get('index_file'));
			$this->location_options_page = new ResponsiveGoogleMaps_MarkAdCoreClasses_Options(dirname(self::$settings->get('index_file')).'/options/location-settings.php', self::$settings->get('index_file'));
			$this->map_type_options_page = new ResponsiveGoogleMaps_MarkAdCoreClasses_Options(dirname(self::$settings->get('index_file')).'/options/map-type-settings.php', self::$settings->get('index_file'));
			$this->cluster_options_page = new ResponsiveGoogleMaps_MarkAdCoreClasses_Options(dirname(self::$settings->get('index_file')).'/options/cluster-settings.php', self::$settings->get('index_file'));
		}
	}
	
	public function add_metaboxes(){
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-shortcode.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-quick-add.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-list.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/location-shortcode.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/location.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/location-info.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/location-map.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-type-preview.php');
		new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-types.php');
		if($this->get_default('plugin_settings', 'allow_override', 0))
			new ResponsiveGoogleMaps_MarkAdCoreClasses_Metabox(dirname(self::$settings->get('index_file')).'/metaboxes/map-settings.php');
	}
	
	public function localize_plugin()
	{
		$localize_map_styles = false;
		$localize_frontend = true;
		$locale = NULL;
		$style_locale = NULL;
		$front_end_mo_file = NULL;
		$map_style_mo_file = NULL;
		
		$map_style_settings = get_option('map_style_settings');
		$map_style_mo_file_dir = trailingslashit(get_template_directory());
		if(is_array($map_style_settings))
		{
			$map_style_mo_file_dir .= $map_style_settings['domain_folder'];
			$map_style_mo_file_dir = trailingslashit($map_style_mo_file_dir);
			$this->map_style_domain = $map_style_settings['text_domain'];
			if(array_key_exists('localize_with_browser', $map_style_settings))
				$localize_map_styles = (bool)$map_style_settings['localize_with_browser'][0];
		}
		
		$plugin_settings = get_option('plugin_settings');
		if(is_array($plugin_settings) && array_key_exists('disable_frontend_localization', $plugin_settings) && $plugin_settings['disable_frontend_localization'][0] == 1)
			$localize_frontend = false;
		
		if(is_admin() && is_array($plugin_settings) && array_key_exists('disable_backend_localization', $plugin_settings) && $plugin_settings['disable_backend_localization'][0] == 1)
			// Delay the unloading because it is not loaded yet
			add_action('init', create_function('$a', 'unload_textdomain("markad-core-text-domain");'));
		else
			// Load standard text domain
			load_plugin_textdomain(self::LANG, false, dirname(plugin_basename(self::$settings->get('index_file'))) . '/languages/back-end' );
		
		// Browser based localization
		if($localize_frontend || $localize_map_styles)
		{
			// Get user langs
			$language_list = $this->parse_language_list();
			if(is_array($language_list) && !empty($language_list))
			{
				// Get the first available .mo file
				foreach($language_list as $priority => $language)
				{
					if(is_array($language))
					{
						foreach($language as $lang)
						{
							if($localize_frontend && $front_end_mo_file === NULL && file_exists(dirname(self::$settings->get('index_file')).'/languages/front-end/'.self::LANG.'-'.$this->parse_language($lang).'.mo')){
								$front_end_mo_file = dirname(self::$settings->get('index_file')).'/languages/front-end/'.self::LANG.'-'.$this->parse_language($lang).'.mo';
								$locale = $lang;
							}
							if($localize_map_styles && $map_style_mo_file === NULL && file_exists($map_style_mo_file_dir.$this->parse_language($lang).'.mo')){
								$map_style_mo_file = $map_style_mo_file_dir.$this->parse_language($lang).'.mo';
								$style_locale = $lang;
							}
						}
					}
					else
					{
						if($localize_frontend && $front_end_mo_file === NULL && file_exists(dirname(self::$settings->get('index_file')).'/languages/front-end/'.self::LANG.'-'.$this->parse_language($language).'.mo')){
							$front_end_mo_file = dirname(self::$settings->get('index_file')).'/languages/front-end/'.self::LANG.'-'.$this->parse_language($language).'.mo';
							$locale = $language;
						}
						if($localize_map_styles && $map_style_mo_file === NULL && file_exists($map_style_mo_file_dir.$this->parse_language($language).'.mo')){
							$map_style_mo_file = $map_style_mo_file_dir.$this->parse_language($language).'.mo';
							$style_locale = $language;
						}
					}
				}
				if($localize_frontend && $front_end_mo_file && $locale)
				{
					$this->front_end_domain = self::LANG.'-'.$locale;
					load_textdomain($this->front_end_domain, $front_end_mo_file);
				}
				if($localize_map_styles && $map_style_mo_file && $style_locale)
				{
					$this->map_style_domain = 'map-style-'.$style_locale;
					load_textdomain($this->map_style_domain, $map_style_mo_file);
				}
			}
		}
		// Front end terms used in JavaScript
		$this->front_end_terms = array(
			'a' => __('A', $this->front_end_domain),
			'auto_fit_map' => __('Auto fit map', $this->front_end_domain),
			'b' => __('B', $this->front_end_domain),
			'clear' => __('Clear', $this->front_end_domain),
			'directions' => __('Directions', $this->front_end_domain),
			'exit_full_screen' => __('Exit full screen', $this->front_end_domain),
			'from_here' => __('from here', $this->front_end_domain),
			'full_screen' => __('Full screen', $this->front_end_domain),
			'get_directions' => __('Get directions', $this->front_end_domain),
			'intext' => __('in', $this->front_end_domain),
			'my_location' => __('My location', $this->front_end_domain),
			'options' => __('Options', $this->front_end_domain),
			'out' => __('out', $this->front_end_domain),
			'resettext' => __('reset', $this->front_end_domain),
			'reset_map' => __('Reset map', $this->front_end_domain),
			'reset_zoom' => __('Reset zoom', $this->front_end_domain),
			'reverse_directions' => __('Reverse directions', $this->front_end_domain),
			'to_here' => __('to here', $this->front_end_domain),
			'undo' => __('Undo', $this->front_end_domain),
			'unsuccessful' => __('Unsuccessful', $this->front_end_domain),
			'you' => __('You', $this->front_end_domain),
			'you_are_here' => __('You are here.', $this->front_end_domain),
			'zoom' => __('Zoom', $this->front_end_domain),
			'zoom_in' => __('Zoom in', $this->front_end_domain),
			'zoom_out' => __('Zoom out', $this->front_end_domain),
		);
	}
	
	public function plugin_activation(){
		// need to register taxonimies and post types
		$this->register_taxonomies();
		$this->create_custom_post_types();
		// flush rewrite rules only once
		flush_rewrite_rules();
	}
	
	public function plugin_deactivation(){
		// For future use
	}
	
	public static function plugin_uninstall(){
		if(!current_user_can('activate_plugins'))
			return;
		
		ResponsiveGoogleMaps_MarkAdCoreClasses_Options::uninstall();
	
		//now delete all map, location, and style posts
		$map_posts = get_posts( array('post_type' => self::$settings->get('map_slug')));
		foreach($map_posts as $post){
			// Delete's each post and meta.
			wp_delete_post( $post->ID, true);
		}
		$location_posts = get_posts( array('post_type' => self::$settings->get('location_slug')));
		foreach($location_posts as $post){
			// Delete's each post and meta.
			wp_delete_post( $post->ID, true);
		}
		$map_type_posts = get_posts( array('post_type' => self::$settings->get('map_type_slug')));
		foreach($map_type_posts as $post){
			// Delete's each post and meta.
			wp_delete_post( $post->ID, true);
		}
	}
	
	// Hooked From - add_filter('plugin_row_meta', array($this, 'set_plugin_meta'), 10, 2);
	public function set_plugin_meta($links, $file){
		$plugin = plugin_basename(self::$settings->get('index_file'));
		if ($file == $plugin)
			return array_merge($links, array( 
					//sprintf( '<a href="post-new.php?post_type='.self::$settings->get('map_slug').'">%s</a>', 'Add a map'),
					sprintf( '<a href="edit.php?post_type='.self::$settings->get('map_slug').'&page=%s">%s</a>', self::$settings->get('map_slug').'-settings', __('Map settings', self::LANG)),
					//sprintf( '<a href="post-new.php?post_type='.self::$settings->get('location_slug').'">%s</a>', 'Add a location'),
					sprintf( '<a href="edit.php?post_type='.self::$settings->get('location_slug').'&page=%s">%s</a>', self::$settings->get('location_slug').'-settings', __('Location settings', self::LANG)),
				)
			);
		return $links;
	}
	
	// Hooked from - add_action('admin_menu', array($this, 'create_menu'));
	public function create_menu($hook){				
		ResponsiveGoogleMaps_MarkAdCoreClasses_Help::AddMenuTab(array("load-post-new.php?post_type=".self::$settings->get('location_slug'), "load-post.php?post_type=".self::$settings->get('location_slug')), __("Location Help", self::LANG), dirname(self::$settings->get('index_file')).'/help/location-pages.html');
		ResponsiveGoogleMaps_MarkAdCoreClasses_Help::AddMenuTab(array("load-post-new.php?post_type=".self::$settings->get('map_slug'), "load-post.php?post_type=".self::$settings->get('map_slug')), __("Map Help", self::LANG), dirname(self::$settings->get('index_file')).'/help/map-pages.html');
		ResponsiveGoogleMaps_MarkAdCoreClasses_Help::AddMenuTab(array("load-post-new.php?post_type=".self::$settings->get('map_type_slug'), "load-post.php?post_type=".self::$settings->get('map_type_slug')), __("Map Style Help", self::LANG), dirname(self::$settings->get('index_file')).'/help/map-styles.html');
		ResponsiveGoogleMaps_MarkAdCoreClasses_Help::AddMenuTab(array("load-post-new.php", "load-post.php"), __("Google Map Shortcodes", self::LANG), dirname(self::$settings->get('index_file')).'/help/map-shortcodes.html');
	}
	
	// Hooked from - add_action('init', array($this, 'register_scripts_and_styles'));
	public function register_scripts_and_styles()
	{
		wp_deregister_style('markad-google-map-plugin-style');
		wp_register_style('markad-google-map-plugin-style', self::$settings->get('plugin_url') . 'css/map-style.css');
		
		wp_deregister_style('markad-google-map-plugin-style-for-vc');
		wp_register_style('markad-google-map-plugin-style-for-vc', self::$settings->get('plugin_url') . 'css/vc-style.css');
		
		wp_deregister_script('markad-google-map-marker-cluster');
		wp_register_script('markad-google-map-marker-cluster', 'https://google-maps-utility-library-v3.googlecode.com/svn/tags/markerclustererplus/2.1.2/src/markerclusterer_packed.js');
		
		wp_deregister_script('markad-google-map-js');
		if(file_exists(dirname(self::$settings->get('index_file')).'/js/map.min.js'))
			wp_register_script('markad-google-map-js', self::$settings->get('plugin_url') . 'js/map.min.js', array('jquery', 'markad-google-map-marker-cluster'));
		else
			wp_register_script('markad-google-map-js', self::$settings->get('plugin_url') . 'js/map.js', array('jquery', 'markad-google-map-marker-cluster'));
		
		wp_deregister_script('markad-google-map-js-admin');
		if(file_exists(dirname(self::$settings->get('index_file')).'/js/map-admin.min.js'))
			wp_register_script('markad-google-map-js-admin', self::$settings->get('plugin_url') . 'js/map-admin.min.js', array('markad-google-map-js', 'jquery', 'markad-google-map-marker-cluster'));
		else
			wp_register_script('markad-google-map-js-admin', self::$settings->get('plugin_url') . 'js/map-admin.js', array('markad-google-map-js', 'jquery', 'markad-google-map-marker-cluster'));
	}
	
	public function register_taxonomies()
	{
		$category_args = array( 
			'hierarchical' => true,
			'label' => __('Categories', self::LANG), 
			'query_var' => true, 
			'rewrite' => true ,
			'show_ui' => true
		);
		// not currently used, maybe in the future
		$tag_args = array(
			'hierarchical' => false,
			'label' => __('Tags', self::LANG),
			'query_var' => true,
			'rewrite' => true,
			'show_ui' => true
		);
		register_taxonomy(self::$settings->get('location_slug').'-category', self::$settings->get('location_slug'), $category_args);
		// may be used in the future
		//register_taxonomy('marker-category', 'attachment', $category_args);
	}
	
	// Custom Post Types
	public function create_custom_post_types()
	{
		$map_labels = array(
			'name' => __('All Google Maps', self::LANG),
			'singular_name' => __('Google Map', self::LANG),
			'add_new' => __('New Google Map', self::LANG),
			'add_new_item' => __('Add New Google Map', self::LANG),
			'edit_item' => __('Edit Google Map', self::LANG),
			'new_item' => __('New Google Map', self::LANG),
			'all_items' => __('All Google Maps', self::LANG),
			'view_item' => __('View Google Map', self::LANG),
			'search_items' => __('Search Google Maps', self::LANG),
			'not_found' => __('No Google Maps found', self::LANG),
			'not_found_in_trash' => __('No Google Maps found in Trash',  self::LANG),
			'parent_item_colon' => '',
			'menu_name' => __('Google Maps', self::LANG)
		);
		
		// Are maps public
		if((bool) self::get_default('plugin_settings', 'public', 0))
			$map_supports = array('title', 'editor');
		else
			$map_supports = array('title'	);
		
		$map_post = register_post_type( 
			self::$settings->get('map_slug'),
			array( 
				'labels' => $map_labels, 
				'_builtin' => false,
				'exclude_from_search' => (bool) self::get_default('plugin_settings', 'searchable', 0), // Exclude from Search Results
				'capability_type' => 'page',
				'public' => (bool) self::get_default('plugin_settings', 'public', 0),
				'publicly_queryable' => (bool) self::get_default('plugin_settings', 'public', 0),
				'query_var' => self::$settings->get('map_slug'),
				'show_ui' => true,
				'show_in_nav_menus' => false,
				'menu_icon' => self::$settings->get('plugin_url').'/images/icon-map.png',
				'menu_position' => 100,
				'rewrite' => true,
				'rewrite' => array(
					'slug' => self::$settings->get('map_slug'),
					'with_front' => false
				),
				'hierarchical' => false,
				'supports' => $map_supports,
			)
		);
		
		$location_labels = array(
			'name' => __('All Locations', self::LANG),
			'singular_name' => __('Location', self::LANG),
			'add_new' => __('New Location', self::LANG),
			'add_new_item' => __('Add New Location', self::LANG),
			'edit_item' => __('Edit Location', self::LANG),
			'new_item' => __('New Location', self::LANG),
			'all_items' => __('All Locations', self::LANG),
			'view_item' => __('View Location', self::LANG),
			'search_items' => __('Search Locations', self::LANG),
			'not_found' =>  __('No Locations found', self::LANG),
			'not_found_in_trash' => __('No Locations found in Trash',  self::LANG),
			'parent_item_colon' => '',
			'menu_name' => __('Map Locations', self::LANG)
		);
		
		// Are locations public
		if((bool) self::get_default('location_settings', 'public', 0))
			$location_supports = array('title','editor');
		else
			$location_supports = array('title');
		
		register_post_type( 
			self::$settings->get('location_slug'),
			array( 
				'labels' => $location_labels, 
				'_builtin' => false,
				'exclude_from_search' => (bool) self::get_default('location_settings', 'searchable', 0), // Exclude from Search Results
				'capability_type' => 'post',
				'public' => (bool) self::get_default('location_settings', 'public', 0),
				'publicly_queryable' => (bool) self::get_default('location_settings', 'public', 0),
				'query_var' => self::$settings->get('location_slug'),
				'show_ui' => true,
				//'show_in_menu' => 'edit.php?post_type='.self::$settings->get('map_slug'),
				'show_in_menu' => true,
				'show_in_nav_menus' => true,
				'menu_icon' => self::$settings->get('plugin_url').'/images/icon-location.png',
				'menu_position' => 100,
				'rewrite' => true,
				'rewrite' => array(
					'slug' => self::$settings->get('location_slug'),
					'with_front' => false
				),
				'hierarchical' => false,
				'supports' => $location_supports,
				'taxonomies' => array(self::$settings->get('location_slug').'-category', self::$settings->get('location_slug').'-tag')
			)
		);
		
		$map_type_labels = array(
			'name' => __('Map Styles', self::LANG),
			'singular_name' => __('Map Style', self::LANG),
			'add_new' => __('New Map Style', self::LANG),
			'add_new_item' => __('Add New Map Style', self::LANG),
			'edit_item' => __('Edit Map Styles', self::LANG),
			'new_item' => __('New Map Style', self::LANG),
			'all_items' => __('All Map Styles', self::LANG),
			'view_item' => __('View Map Style', self::LANG),
			'search_items' => __('Search Map Styles', self::LANG),
			'not_found' =>  __('No Map Style found', self::LANG),
			'not_found_in_trash' => __('No Map Style found in Trash',  self::LANG),
			'parent_item_colon' => '',
			'menu_name' => __('Map Styles', self::LANG),
		);
		
		$map_type_supports = array('title');

		register_post_type( 
			self::$settings->get('map_type_slug'),
			array( 
				'labels' => $map_type_labels, 
				'_builtin' => false,
				'exclude_from_search' => true, // Exclude from Search Results
				'capability_type' => 'post',
				'public' => false,
				'publicly_queryable' => false,
				'query_var' => self::$settings->get('map_type_slug'),
				'show_ui' => true,
				//'show_in_menu' => 'edit.php?post_type='.self::$settings->get('map_slug'),
				'show_in_menu' => true,
				'show_in_nav_menus' => true,
				'menu_icon' => self::$settings->get('plugin_url').'/images/icon-map-style.png',
				'menu_position' => 100,
				'rewrite' => true,
				'rewrite' => array(
					'slug' => self::$settings->get('map_type_slug'),
					'with_front' => false
				),
				'hierarchical' => false,
				'supports' => $map_type_supports,
			)
		);
	}
	
	private function check_for_vc_shortcode($content)
	{
		if(strpos($content, '[google_map_vc map='))
			return true;
		else
			return false;
	}
	
	public function check_for_shortcode()
	{
		if(function_exists('has_shortcode'))
		{
			global $post;
			if(is_object($post) && (has_shortcode($post->post_content, 'google_map') || $this->check_for_vc_shortcode($post->post_content)))
				wp_enqueue_style('markad-google-map-plugin-style');
		}
	}
	
	public function add_scripts_and_styles()
	{		
		wp_enqueue_style('markad-google-map-plugin-style');	
		wp_enqueue_script('markad-google-map-js');
		if(!self::$global_settings_echoed)
		{
			wp_localize_script('markad-google-map-js', 'global_map_settings', $this->get_global_map_settings());
			wp_localize_script('markad-google-map-js', 'RGMLocal', $this->front_end_terms);
			if(is_admin()){
				add_action('admin_footer', array($this, 'output_available_styles'), 0);
			} else {
				add_action('wp_footer', array($this, 'output_available_styles'), 0);
			}
			self::$global_settings_echoed = true;
		}
	}
	
	// Hooked from - add_action('admin_enqueue_scripts', array($this, 'add_admin_scripts_and_styles'));
	public function add_admin_scripts_and_styles($hook = NULL)
	{
		wp_enqueue_style('markad-google-map-plugin-style');
		
		$ajaxurl = admin_url( 'admin-ajax.php' );
		if($_SERVER['QUERY_STRING']) 
			$ajaxurl .= '?'.$_SERVER['QUERY_STRING'];
		
		global $post;

		if($hook == $this->map_options_page->menu_hook)
		{
			wp_enqueue_script('markad-google-map-js');
			wp_enqueue_script('markad-google-map-js-admin');
			wp_localize_script('markad-google-map-js', 'global_map_settings', $this->get_global_map_settings());
			wp_localize_script('markad-google-map-js', 'map_page_admin_settings', array('autocenter' => 0, 'update_default_center' => true, 'directions' => 0, 'show_user' => 0, 'location_page' => 1));
			wp_localize_script('markad-google-map-js', 'RGMLocal', $this->front_end_terms);
			$callback = create_function('$a', 'echo \'<script type="text/javascript">jQuery(document).ready(function(){new ResponsiveGoogleMap("map_canvas");});</script>\';');
			add_action('admin_footer', $callback);
		}
		else if((get_post_type($post) == self::$settings->get('map_slug') && ($hook == 'post.php' || $hook == 'post-new.php')))
		{
			wp_enqueue_script('markad-google-map-js');
			wp_enqueue_script('markad-google-map-js-admin');
			wp_localize_script('markad-google-map-js', 'global_map_settings', $this->get_global_map_settings());
			wp_localize_script('markad-google-map-js', 'map_locations', $this->get_locations_by_ids($this->get_locations_for_map()));
			wp_localize_script('markad-google-map-js', 'ind_map_settings', $this->get_settings_for_indvidual_map());
			wp_localize_script('markad-google-map-js', 'RGMLocal', $this->front_end_terms);
			$this->output_available_styles();
			if($hook == 'post-new.php')
				wp_localize_script('markad-google-map-js', 'map_page_admin_settings', array('draggable' => false, 'update_default_center' => true, 'autocenter' => 0, 'show_user' => 0));
			else
				wp_localize_script('markad-google-map-js', 'map_page_admin_settings', array('draggable' => false, 'update_default_center' => true));
			$callback = create_function('$a', 'echo \'<script type="text/javascript">jQuery(document).ready(function(){var markad_google_map = new ResponsiveGoogleMap("map_canvas", map_locations, ind_map_settings);});</script>\';');
			add_action('admin_footer', $callback);
		}
		if((get_post_type($post) == self::$settings->get('location_slug') && ($hook == 'post.php' || $hook == 'post-new.php')))
		{
			wp_enqueue_script('markad-google-map-js');
			wp_enqueue_script('markad-google-map-js-admin');
			wp_localize_script('markad-google-map-js', 'global_map_settings', $this->get_global_map_settings());
			wp_localize_script('markad-google-map-js', 'map_locations', $this->get_single_locations_for_map());
			wp_localize_script('markad-google-map-js', 'map_page_admin_settings', array('autocenter' => 1, 'draggable' => true, 'directions' => 0, 'location_page' => 1, 'show_user' => 0));
			wp_localize_script('markad-google-map-js', 'RGMLocal', $this->front_end_terms);
			$callback = create_function('$a', 'echo \'<script type="text/javascript">jQuery(document).ready(function(){new ResponsiveGoogleMap("map_canvas", map_locations);});</script>\';');
			add_action('admin_footer', $callback);
		}
		if((get_post_type($post) == self::$settings->get('map_type_slug') && ($hook == 'post.php' || $hook == 'post-new.php')))
		{
			wp_enqueue_script('markad-google-map-js');
			wp_enqueue_script('markad-google-map-js-admin');
			wp_localize_script('markad-google-map-js', 'global_map_settings', $this->get_global_map_settings());
			wp_localize_script('markad-google-map-js', 'map_page_admin_settings', array('autocenter' => 1, 'draggable' => true, 'directions' => 0, 'location_page' => 1, 'show_user' => 0));
			wp_localize_script('markad-google-map-js', 'RGMLocal', $this->front_end_terms);
			$style = $this->get_map_custom_style($post->ID);
			if(is_string($style))
				$style = json_decode($style);
			wp_localize_script('markad-google-map-js', 'custom_style', $style);
			$callback = create_function('$a', 'echo \'<script type="text/javascript">var map_settings = {"map_type":["ROADMAP"],"enable_zoom_control":["1"],"disable_doubleclick_zoom":["0"],"enable_scroll_wheel_zoom":["1"],"tilt":["45"],"autocenter":["1"],"enable_directions":["1"],"show_overview_map_control":["1"],"overview_map_control_opened":["1"],"enable_scale_control":["1"],"enable_rotate_control":["1"],"enable_pan_control":["1"],"enable_street_view_control":["1"],"enable_map_type_control":["0"],"enable_autofit_button":["0"],"enable_map_reset_button":["0"],"enable_fullscreen_view":["1"],"info_window_max_width":"350","map_bg_color":"#ffffff","enable_map_nav_color":["0"],"show_user_location":["0"],"enable_marker_clustering":["0"],"enable_weather_layer":["0"],"weather_layer_temp":["Celsius"],"weather_layer_wind_speed":["Meters Per Second"],"enable_cloud_layer":["0"],"enable_panoramio_layer":["0"],"panoramio_filter_id":"","panoramio_filter_tag":"","enable_bicycle_layer":["0"],"enable_traffic_layer":["0"],"enable_transit_layer":["0"],"map_api_key":"","fade_speed":"500","allow_override":["1"]};jQuery(document).ready(function(){new ResponsiveGoogleMap("map_canvas",null,map_settings);});</script>\';');
			add_action('admin_footer', $callback);
		}
	}
	
	public function set_single_template($single)
	{
		global $wp_query, $post;
		/* Checks for single template by post type */
		if ($post->post_type == self::$settings->get('location_slug'))
		{
			if(file_exists(get_template_directory() . '/single-'.self::$settings->get('location_slug').'.php'))
				return get_template_directory() . '/single-'.self::$settings->get('location_slug').'.php';
			else if(file_exists(self::$settings->get('template_directory') . '/single-'.self::$settings->get('location_slug').'.php'))
				return self::$settings->get('template_directory') . '/single-'.self::$settings->get('location_slug').'.php';
		}
		else if ($post->post_type == self::$settings->get('map_slug'))
		{
			if(file_exists(get_template_directory() . '/single-'.self::$settings->get('map_slug').'.php'))
				return get_template_directory() . '/single-'.self::$settings->get('map_slug').'.php';
			else if(file_exists(self::$settings->get('template_directory') . '/single-'.self::$settings->get('map_slug').'.php'))
				return self::$settings->get('template_directory') . '/single-'.self::$settings->get('map_slug').'.php';
		}
		return $single;
	}
	
	public function set_taxonomy_template($category)
	{
		global $wp_query, $post;
		/* Checks for single template by post type */
		//if (is_object($post) && $post->post_type == self::$settings->get('location_slug'))
		if ($wp_query->query_vars['taxonomy'] == self::$settings->get('location_slug').'-category')
		{
			if(file_exists(get_template_directory() . '/taxonomy-'.self::$settings->get('location_slug').'-category.php'))
				return get_template_directory() . '/taxonomy-'.self::$settings->get('location_slug').'-category.php';
			else if(file_exists(self::$settings->get('template_directory') . '/taxonomy-'.self::$settings->get('location_slug').'-category.php'))
				return self::$settings->get('template_directory') . '/taxonomy-'.self::$settings->get('location_slug').'-category.php';
		}
		return $category;
	}
	
	public function map_columns_head($defaults){
		$date = $defaults['date'];
		unset($defaults['date']);
		$defaults['shortcode'] = __('Shortcode', self::LANG);
		$defaults['date'] = $date;
		return $defaults;
	}
	
	public function map_columns_content($column_name, $post_id){
    	if($column_name == 'shortcode')
			echo '[google_map map_id="'.$post_id.'"/]';
	}
	
	public function location_columns_head($defaults){
		$date = $defaults['date'];
		unset($defaults['date']);
		
		$defaults['shortcode'] = __('Shortcode', self::LANG);
		$defaults['category'] = __('Categories', self::LANG);
		$defaults['marker_image'] = __('Marker', self::LANG);
		$defaults['geocode_address'] = __('Address', self::LANG);
		$defaults['lat_lng'] = __('Lat / Lng', self::LANG);
		$defaults['date'] = $date;
		
		return $defaults;
	}

	public function location_columns_content($column_name, $post_id){
		$map_location = (array)get_post_meta($post_id, 'map_location', true);
		if($column_name == 'shortcode')
			echo '[google_map location_id="'.$post_id.'"/]';

    	if($column_name == 'category'){  
			$terms = wp_get_post_terms($post_id, self::$settings->get('location_slug').'-category');
			if(is_array($terms)){
				foreach($terms as $term)
					echo $term->name.'<br />';
			}
		}
		if($column_name == 'marker_image'){
			if(array_key_exists('image', $map_location) && $map_location['image'])
				$post_marker_image = $map_location['image'];
			else
				$post_marker_image = $this->get_default('default_image');
			
			if($post_marker_image)
				echo '<img src="' . $post_marker_image . '" style="width:auto; height:30px;"" />';
		}
		if($column_name == 'geocode_address'){
			if(array_key_exists('geocode_address', $map_location) && $map_location['geocode_address'])
				echo $map_location['geocode_address'];
		}
		if($column_name == 'lat_lng'){
			$address = '';
			if(array_key_exists('lat', $map_location) && $map_location['lat'] && array_key_exists('lng', $map_location) && $map_location['lng'])
				$address = $map_location['lat'] . ',<br />'. $map_location['lng'];
			
			if($address)
				echo $address;
		}
	}
	
	// Media strings for the custom map marker media manager
	public function custom_media_string($strings){
		$strings['customMenuTitle'] = __('Map Markers', self::LANG);
		$strings['customButton'] = __('Use This Marker', self::LANG);
		$strings['markerURLButton'] = __('Use This Image', self::LANG);
		return $strings;
	}
	
	// Override templete for insert from URL in custom map marker media manager
	public function print_marker_override_template(){
		echo '<script type="text/html" id="tmpl-embed-image-settings-override">';
		echo '   <div class="thumbnail">';
		echo '      <img src="{{ data.model.url }}" draggable="false" />';
		echo '   </div>';
		echo '</script>';
	}
	
	// AJAX call to load map markers in media select
	public function add_media_marker_tab_content(){
		$filenames = array();
		if($handle = opendir(self::$settings->get('marker_directory'))){
			while(false !== ($entry = readdir($handle))){
				if($entry && $entry != "." && $entry != ".." && substr($entry, 0, 1) != ".") //don't show hidden files
					$filenames[] = $entry;
			}
			closedir($handle);
			asort($filenames);
			echo '<div class="custom_marker_select"><ul class="attachments ui-sortable ui-sortable-disabled" id="map_makers_list">';
			foreach($filenames as $file){
				$url = self::$settings->get('marker_url') . $file;
				echo '<li class="attachment save-ready marker-thumbnail">';
				echo '<div class="attachment-preview type-image subtype-png portrait">';
				echo '	<div class="thumbnail" data-src="'.$url.'">';
				echo '		<div class="centered">';
				echo '			<img src="'.$url.'" draggable="false">';
				echo '		</div>';
				echo '	</div>';
				//echo '	<a class="check" href="#" title="Deselect"><div class="media-modal-icon"></div></a>';
				echo '</div>';
				echo '</li>';
			}
			echo '</ul></div>';
		}
		die();
	}
 	
	// AJAX function for adding new locations to the map
	public function ajax_add_location_to_map($post_id = NULL){
		if($post_id){
			$location = array(
				'status' => 1,
				'location' => $this->get_single_locations_for_map($post_id)
			);
		}
		if(!$post_id && isset($_POST['location_id'])){
			$location = array(
				'status' => 1,
				'location' => $this->get_single_locations_for_map((int) $_POST['location_id'])
			);
		}
		else {
			$location = array(
				'status' => 0,
				'message' => __('There was a problem getting that location. Please try again later.', self::LANG)
			);	
		}
		header("Content-type: application/javascript;");
		echo json_encode($location);
		die();
	}
	
	// AJAX function to add a new location post from the quick add metabax
	public function ajax_quick_add_location_to_map(){
		$post_args = array(
			'post_title'    => $_POST['location']['name'],
			'post_type'     => self::$settings->get('location_slug'),
			'post_status'   => 'publish',
		);
		$post_id = wp_insert_post($post_args);
		
		if($post_id){
			update_post_meta($post_id, 'map_location', $_POST['location']);
			update_post_meta($post_id, 'location_info', array('lat' => $_POST['location']['lat'], 'lng' => $_POST['location']['lng']));
			
			$response = array(
				'status' => 1,
				'id' => $post_id
			);
		}
		else
			$response = array(
				'status' => 0,
				'message' => 'error'
			);
			
		echo json_encode($response);
		die();
	}
	
	public static function get_metabox_list_of_locations(){
		$return = array();
		$args = array(
			'numberposts'     => 0,
			'orderby'         => 'title',
			'order'           => 'ASC',
			'post_type'       => self::$settings->get('location_slug'),
			'post_status'     => 'publish',
			'posts_per_page'	=> -1,
			'suppress_filters' => true 
		);
		
		$posts_array = get_posts( $args );
		foreach($posts_array as $post){
			$return[] = array(
				'id' => $post->ID,
				'label' => $post->post_title,
				'value' => true,
				'default' => false,
				'attributes' => array(
					'class' => 'map-locations-checkbox'
				)
			);
		}
		return $return;
	}
	
	public function get_single_locations_for_map($post_id = NULL)
	{
		if($post_id === NULL)
		{
			global $post;
			if(!is_object($post))
				return;
			$post_id = $post->ID;
		}
		
		if(get_post_status($post_id) == 'publish' && is_array($location = get_post_meta($post_id, 'map_location', true)))
		{
			$location['id'] = $post_id;
			$location['name'] = get_the_title($post_id);
			//$content = apply_filters('the_content', $location['infowindow_content']);
			if($location['infowindow_content'])
				$content = apply_filters('infowindow_content', wpautop($location['infowindow_content'], true), $location['id']);
			else
				$content = wpautop($location['geocode_address'], true);
			$location['infowindow_content'] = str_replace(']]>', ']]&gt;', $content);

			if(!array_key_exists('image', $location) || !$location['image'])
				$location['image'] = self::get_default('location_settings', 'default_image');
			
			return $location;
		}
		return NULL;
	}
	
	public function get_locations_for_map($post = NULL)
	{
		if($post === NULL)
			global $post;
		$locations = array();
		if(is_object($post))
		{
			$map_locations = get_post_meta($post->ID, 'map_locations', true);
			if(is_array($map_locations) && array_key_exists('location_ids', $map_locations))
			{
				foreach($map_locations['location_ids'] as $location_id => $location_active)
				{
					if(get_post_status($location_id) == 'publish' && $location_active)
						$locations[] = $location_id;
				}
			}
		}
		return $locations;
	}
	
	public function get_category_locations_for_map($categories, $field = 'name')
	{
		$locations = array();
		$categories = explode(',',$categories);
		foreach($categories as $category)
		{
			$args = array(
				'posts_per_page' => 0,
				'post_type' => self::$settings->get('location_slug'),
				'post_status' => 'publish',
				'tax_query' => array(
					array(
						'taxonomy' => self::$settings->get('location_slug').'-category',
						'field' => $field,
						'terms' => $category
					)
				)
			);
			$posts = get_posts($args);
			foreach($posts as $post)
			{
				if(!in_array($this->get_single_locations_for_map($post->ID), $locations))
					$locations[] = $post->ID;
			}
		}
		return $locations;
	}
	
	public function get_global_map_settings(){
		$map_settings = get_option('map_settings');
		$plugin_settings = get_option('plugin_settings');
		$cluster_settings = get_option('marker_cluster_settings');
		
		// array keys to unset from plugin_settings so they don't get output to the screen
		$unset_array_keys = array(
			'disable_backend_localization',
			'disable_frontend_localization',
			'public',
			'searchable',
			'allow_override');
		
		foreach($unset_array_keys as $key){
			if(array_key_exists($key, $plugin_settings))
				unset($plugin_settings[$key]);
		}
		
		$cluster_images = array();
		if(is_array($cluster_settings)){
			foreach($cluster_settings as $key => $value){
				if(is_int($key))
					array_push($cluster_images, $value);
			}
		}
		if(empty($cluster_images))
			$cluster_images = false;
		
		if(is_array($location_settings) && is_array($plugin_settings)){
			$unset_location_array_keys = array(
				'public',
				'searchable',
				'async_locations'
			);
			
			foreach($unset_location_array_keys as $key){
				if(array_key_exists($key, $location_settings))
					unset($location_settings[$key]);
			}
		
			$plugin_settings = array_merge($location_settings, $plugin_settings);
		}
		
		if(is_array($map_settings) && is_array($plugin_settings))
			return array_merge($map_settings, $plugin_settings, array(
				'is_admin' => is_admin(),
				'ajaxurl' => admin_url('admin-ajax.php'),
				'cluster_images' => $cluster_images,
			));
		else
			return false;
	}
	
	protected function get_map_custom_style($post_id = NULL)
	{
		$styles = get_post_meta($post_id, 'custom_map_type', true);
		$output_styles = array();
		if(is_array($styles))
		{
			if(array_key_exists('enable_custom_json', $styles) && array_key_exists('custom_json', $styles) && $styles['enable_custom_json'][0] == 1)
			{
				$output_styles = $styles['custom_json'];
				$output_styles = str_replace(array("\r","\n","\r\n","\t"," "), '', $output_styles);
			}
			else
			{
				if(array_key_exists('enable_changes_to', $styles))
				{
					foreach($styles as $style_name => $style)
					{
						$style_type = explode('|', $style_name);
						if(is_array($style_type) && count($style_type) >= 3)
						{
							if(in_array($style_type[0], $styles['enable_changes_to']))
							{
								$inserted = false;
								if(is_array($style))
									$style = $style[0];
								
								if(is_numeric($style))
									$style = (float)$style;
									
								$temp_style = array(
									'featureType' => $style_type[0],
									'elementType' => $style_type[1],
									'stylers' => array(
										array(
											$style_type[2] => $style
										),
									)
								);
								for($i=0;$i<count($output_styles);$i++)
								{
									if($output_styles[$i]['featureType'] == $temp_style['featureType'] &&
										$output_styles[$i]['elementType'] == $temp_style['elementType'])
									{
										$inserted = true;
										array_push($output_styles[$i]['stylers'], array($style_type[2] => $style));
										break;
									}
								}
								if(!$inserted)
									$output_styles[] = $temp_style;
							}
						}
					}
				}
			}
		}
		return $output_styles;
	}
	
	public function get_settings_for_indvidual_map($post = NULL)
	{
		$global_settings = $this->get_global_map_settings();
		if(!$this->get_default('plugin_settings', 'allow_override', 0))
			return $global_settings;
		
		if($post === NULL)
			global $post;
			
		$settings = array();
		if(is_object($post))
			$settings = get_post_meta($post->ID, 'map_settings', true);
		else
			$settings = get_post_meta($post, 'map_settings', true);

		if(is_array($settings) && array_key_exists('enable_overrides', $settings) && $settings['enable_overrides'][0] == 1)
		{
			if(is_numeric($settings['map_type'][0]))
			{
				$this->add_style_to_output_array($settings['map_type'][0]);
				$this->add_style_id_to_external_source($settings['map_type'][0]);
			}
			if(array_key_exists('enable_map_types', $settings) && is_array($settings['enable_map_types']))
			{
				foreach($settings['enable_map_types'] as $style_id)
				{
					$this->add_style_to_output_array($style_id);
					$this->add_style_id_to_external_source($style_id);
				}
				if(is_numeric($settings['map_type'][0]) && !in_array($settings['map_type'][0], $settings['enable_map_types']))
					array_push($settings['enable_map_types'], $settings['map_type'][0]);
			}
			else if(is_numeric($settings['map_type'][0]) && !array_key_exists('enable_map_types', $settings))
				$settings['enable_map_types'] = array((string)$settings['map_type'][0]);
			else
				$settings['enable_map_types'] = array();
				
			return $settings;
		}
		else
			return $global_settings;
	}
	
	protected function add_style_id_to_external_source($id)
	{
		if(!in_array($id, $this->external_style_ids))
			array_push($this->external_style_ids, $id);
	}
	
	protected function add_style_to_output_array($post_id)
	{
		$post = get_post($post_id);
		if(is_object($post) && $post->post_status == 'publish')
		{
			if(!array_key_exists($post->ID, $this->available_styles_array))
			{
				$style = $this->get_map_custom_style($post->ID);
				if(is_string($style))
					$style = json_decode($style);
							
				$this->available_styles_array[$post->ID] = array(
					'id' => (string)$post->ID,
					'name' => __($post->post_title, $this->map_style_domain),
					'style' => $style
				);
			}
		}
	}
	
	public function ajax_get_available_styles(){
		if(isset($_POST['styles']))
			$styles = (array)$_POST['styles'];
		else if(isset($_GET['styles']))
			$styles = (array)$_GET['styles'];
		else
			$styles = array();
		foreach($styles as $style)
			$this->add_style_to_output_array($style);
			
		$plugin_settings = get_option('plugin_settings');
		if(is_array($plugin_settings) && array_key_exists('enable_front_ajax_cache', $plugin_settings) && $plugin_settings['enable_front_ajax_cache'][0] == 1)
		{
			$expires = 60 * 60;        // 60 minutes
			header('Pragma: public');
			header('Cache-Control: maxage=' . $expires);
			header('Expires: ' . gmdate('D, d M Y H:i:s', time() + $expires) . ' GMT');
		}
		header("Content-type: application/javascript;");

		echo json_encode(array('status' => 1, 'styles' => $this->available_styles_array));
		die();
	}
	
	public function get_locations_by_ids($ids){
		$output = array();
		if(is_array($ids)){
			foreach($ids as $id)
				$output[] = $this->get_single_locations_for_map($id);
		}
		else if(is_numeric($ids) || is_string($ids))
			$output[] = $this->get_single_locations_for_map($ids);			
		return $output;
	}
	
	public function ajax_get_locations_by_ids(){
		if(isset($_POST['ids']))
			$ids = (array)$_POST['ids'];
		else if(isset($_GET['ids']))
			$ids = (array)$_GET['ids'];
		else
			$ids = array();
			
		$output = $this->get_locations_by_ids($ids);
		
		$plugin_settings = get_option('plugin_settings');
		if(is_array($plugin_settings) && array_key_exists('enable_front_ajax_cache', $plugin_settings) && $plugin_settings['enable_front_ajax_cache'][0] == 1)
		{
			$expires = 60 * 60;        // 60 minutes
			header('Pragma: public');
			header('Cache-Control: maxage=' . $expires);
			header('Expires: ' . gmdate('D, d M Y H:i:s', time() + $expires) . ' GMT');
		}
		header("Content-type: application/javascript;");
		echo json_encode(array('status' => 1, 'locations' => $output));
		die();
	}
	
	public function output_available_styles(){
		global $wp_scripts;
		$output_array = array();
		foreach($this->available_styles_array as $style)
			array_push($output_array, $style);
		
		$wp_scripts->localize('markad-google-map-js', 'available_map_styles', $this->external_style_ids);
	}
	
	public static function get_default($option = NULL, $value = NULL, $index = NULL){
		if(!$option)
			return false;
			
		$defaults = get_option($option);
		
		if(!is_array($defaults))
			return false;
		if(array_key_exists($value, $defaults)){
			if(is_array($defaults[$value]) && $index !== NULL){
				if(array_key_exists($index, $defaults[$value]))
					return $defaults[$value][$index];
				else
					return false;
			}
			else
				return $defaults[$value];
		}
		else
			return false;
	}
	
	// SHORTCODE
	/*
	*   Visual Composer block shortcode
	*	[google_map_vc/] shortcode
	*/
	public function google_map_vc($atts)
	{
		extract( shortcode_atts( array(
			'map' => '',
			'height' => '',
			'width' => ''
		), $atts ) );
		$map = explode('|', $map);
		if(count($map) === 2)
		{
			$shortcode = '[google_map '.$map[0].'="'.$map[1].'"';
			if($height)
				$shortcode .= ' height="'.$height.'"';
			if($width)
				$shortcode .= ' width="'.$width.'"';
			$shortcode .= '/]';
			return do_shortcode($shortcode);
		}
	}
	/*
	*	[google_map/] shortcode
	*/
	
	public function google_map( $atts , $content = NULL){
		extract( shortcode_atts( array(
			'height' => '500px',
			'width' => '100%',
			'map_id' => 0,
			'location_id' => 0,
			'category_name' => '',
			'category_slug' => '',
			'category_id' => ''
		), $atts ) );
		
		$unique_id = uniqid('map');	
		
		if($map_id)
			$locations = $this->get_locations_for_map(get_post($map_id));
		else if($location_id)
			$locations = array($location_id);
		else if($category_name)
			$locations = $this->get_category_locations_for_map($category_name, 'name');
		else if($category_slug)
			$locations = $this->get_category_locations_for_map($category_slug, 'slug');
		else if($category_id)
			$locations = $this->get_category_locations_for_map($category_id, 'id');
		else
			return;
		
		$location_settings = get_option('location_settings');
		if(array_key_exists('async_locations', $location_settings) && $location_settings['async_locations'][0] == 1)
			$locations = array('async' => 1, 'locations' => $locations);
		else
			$locations = $this->get_locations_by_ids($locations);
			
		global $wp_scripts;
		$wp_scripts->localize('markad-google-map-js', $unique_id.'_map_locations', $locations);
		
		$settings_id = $map_id ? $map_id : $location_id;
		$ind_map_settings = $this->get_settings_for_indvidual_map($settings_id);
		
		$this->add_scripts_and_styles();	
		
		$output = '';
		$output .= '<div id="'.$unique_id.'_container" class="small_map_container" style="height:'.$height.'; width:'.$width.';">';
		$output .= '<div id="'.$unique_id.'" class="map_canvas"></div>';
		$output .= '</div>';
					
		$output_script = create_function('$a', 'echo \'<script type="text/javascript">jQuery(document).ready(function(){new ResponsiveGoogleMap("'.$unique_id.'", '.$unique_id.'_map_locations, '.json_encode($ind_map_settings).');});</script>\';');
		add_action('wp_footer', $output_script, 100);
		
		/*if(has_action('wp_footer', array($this, 'output_available_styles'))){
			remove_action('wp_footer', array($this, 'output_available_styles'), 10);
			add_action('wp_footer', array($this, 'output_available_styles'), 10);
		}
		else
			add_action('wp_footer', array($this, 'output_available_styles'), 10);*/
		
		return $output;
	}
	
	public function get_map_type_options($custom_only = false){
		//get map type posts
		$args = array(
			'numberposts'     => 0,
			'orderby'         => 'title',
			'order'           => 'ASC',
			'post_type'       => self::$settings->get('map_type_slug'),
			'post_status'     => 'publish',
			'posts_per_page'	=> -1,
			'suppress_filters' => true 
		);
		$posts_array = get_posts( $args );
		$custom_map_types = array();
		if(is_array($posts_array)){
			foreach($posts_array as $post){
				$custom_map_types[] = array(
					'id' => $post->ID,
					'label' => __($post->post_title, self::LANG),
					'value' => $post->ID,
					'text' => $post->post_title,
					'default' => false
				);
			}
		}
		if($custom_only)
			return $custom_map_types;
			
		$map_types = array(
			array(
				'id' => 'roadmap',
				'label' => __('Roadmap', self::LANG),
				'value' => 'ROADMAP',
				'text' => 'Roadmap',
				'default' => true
			),
			array(
				'id' => 'satellite',
				'label' => __('Satellite', self::LANG),
				'value' => 'SATELLITE',
				'text' => 'Satellite',
				'default' => false,
			),
			array(
				'id' => 'hybrid',
				'label' => __('Hybrid', self::LANG),
				'value' => 'HYBRID',
				'text' => 'Hybrid',
				'default' => false,
			),
			array(
				'id' => 'terrain',
				'label' => __('Terrain', self::LANG),
				'value' => 'TERRAIN',
				'text' => 'Terrain',
				'default' => false,
			)
		);
		$map_types = array_merge($map_types, $custom_map_types);
		return $map_types;
	}
	
	// parse list of comma separated language tags and sort it by the quality value
	private function parse_language_list($language_list = NULL){
		if(is_null($language_list)){
			if(!isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
				return array();
			$language_list = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
		}
		$languages = array();
		$language_ranges = explode(',', trim($language_list));
		foreach ($language_ranges as $language_range){
			if (preg_match('/(\*|[a-zA-Z0-9]{1,8}(?:-[a-zA-Z0-9]{1,8})*)(?:\s*;\s*q\s*=\s*(0(?:\.\d{0,3})|1(?:\.0{0,3})))?/', trim($language_range), $match)){
				if (!isset($match[2]))
					$match[2] = '1.0';
				else
					$match[2] = (string) floatval($match[2]);
				if (!isset($languages[$match[2]]))
					$languages[$match[2]] = array();
				$languages[$match[2]][] = strtolower($match[1]);
			}
		}
		krsort($languages);
		return $languages;
	}
	
	private function parse_language($language){
		$language_str = explode('-',$language);
		if(count($language_str) > 1)
			return $language_str[0].'_'.strtoupper($language_str[1]);
		else
			return $language_str[0];
	}
	
	private function get_vc_google_map_pages(){
		$args = array(
			'numberposts'     => 0,
			'orderby'         => 'title',
			'order'           => 'ASC',
			'post_type'       => self::$settings->get('map_slug'),
			'post_status'     => 'publish',
			'posts_per_page'	=> -1,
			'suppress_filters' => true 
		);
		$return = array();
		$posts_array = get_posts($args);
		if(is_array($posts_array) && !empty($posts_array)){
			$return['----- Maps -----'] = '';
			foreach($posts_array as $post){
				$return[$post->post_title.' #'.$post->ID] = 'map_id|'.$post->ID;
			}
		}
		return $return;
	}
	
	private function get_vc_location_pages(){
		$args = array(
			'numberposts'     => 0,
			'orderby'         => 'title',
			'order'           => 'ASC',
			'post_type'       => self::$settings->get('location_slug'),
			'post_status'     => 'publish',
			'posts_per_page'	=> -1,
			'suppress_filters' => true 
		);
		$return = array();
		$posts_array = get_posts($args);
		if(is_array($posts_array) && !empty($posts_array)){
			$return['----- Single Locations -----'] = '';
			foreach($posts_array as $post){
				$return[$post->post_title.' #'.$post->ID] = 'location_id|'.$post->ID;
			}
		}
		return $return;
	}
	
	private function get_vc_category_pages(){
		$taxonomy = self::$settings->get('location_slug').'-category';
		$tax_terms = get_terms($taxonomy);
		$return = array();
		
		if(is_array($tax_terms) && !empty($tax_terms)){
			if(!empty($tax_terms))
				$return['----- Category Maps -----'] = '';
			foreach($tax_terms as $term){
				$return[$term->name.' #'.$term->term_id] = 'category_id|'.$term->term_id;
			}
		}
		return $return;
	}
	
	private function get_vc_map_selections()
	{
		$return = array('None' => '');
		$return = array_merge($return, $this->get_vc_google_map_pages());
		$return = array_merge($return, $this->get_vc_location_pages());
		$return = array_merge($return, $this->get_vc_category_pages());
		return $return;
	}
	
	public function make_visual_composer_block()
	{
		if(function_exists('vc_map'))
		{
			wp_enqueue_style("markad-google-map-plugin-style-for-vc");
			vc_map(array(
				"name" => __("Responsive Google Map", self::LANG),
				"base" => "google_map_vc",
				"icon" => "icon-wpb-google_map_vc",
				"category" => __('Content', self::LANG),
				"description" => __('Responsive Google Maps map block', self::LANG),
				'class' => 'wpb_google_map_vc',
				"params" => array(
					array(
						"type" => "dropdown",
						"heading" => __("Map", self::LANG),
						"param_name" => "map",
						"value" => $this->get_vc_map_selections(),
						"description" => __("Select a map to display.", self::LANG),
						"admin_label" => true
					),
					array(
						"type" => "textfield",
						"heading" => __("Height", self::LANG),
						"param_name" => "height",
						"description" => __("Please include units (500px, 100%) or leave blank for default.", self::LANG)
					),
					array(
						"type" => "textfield",
						"heading" => __("Width", self::LANG),
						"param_name" => "width",
						"description" => __("Please include units (500px, 100%) or leave blank for default.", self::LANG)
					),
				)
			));
		}
	}
	
	public function update_cluster_options($new_value, $old_value)
	{
		foreach($new_value as $key => $value){
			$index = intval(substr($key, strlen($key)-1, 1));
			$new_value[$index] = array();
			$new_value[$index]['index'] = $index;
			
			if(!$value)
				$value = 'https://google-maps-utility-library-v3.googlecode.com/svn/trunk/markerclustererplus/images/m'.$index.'.png';
			$size = @getimagesize($value);
			if($size){	
				$new_value[$index]['url'] = $value;
				$new_value[$index]['width'] = $size[0];
				$new_value[$index]['height'] = $size[1];
			}
		}
		return $new_value;
	}
}
?>