(function(bind_events, $, CS, L10){
	ResponsiveGoogleMap.prototype.geocode_single = function(address, callback){
		var instance = this;
		this.remove_all_markers();
		this.geocode_address(address, function(){ResponsiveGoogleMap.update_position(instance.marker_array[0]);});
	};
	ResponsiveGoogleMap.prototype.geocode_address = function(address, callback){
		var instance = this;
		this.geocoder.geocode({'address': address}, function(results, status){
			if (status == google.maps.GeocoderStatus.OK){
				var geocode_location = {
					geocode_address : results[0].formatted_address,
					infowindow_open : [0],
					infowindow_content : results[0].formatted_address,
					lat : results[0].geometry.location.lat(),
					lng : results[0].geometry.location.lng(),
					id : ResponsiveGoogleMap.generate_uid(),
					name : results[0].formatted_address
				};
				instance.place_marker(geocode_location);
				instance.fit_bounds(true);
				callback();
			}
			else
				alert(L10.unsuccessful+": "+status);
		});
	};
	ResponsiveGoogleMap.prototype.quick_add_geocode_address = function(address, callback){
		var instance = this;
		this.geocoder.geocode({'address': address}, function(results, status){
			if (status == google.maps.GeocoderStatus.OK){
				var geocode_location = {
					geocode_address : results[0].formatted_address,
					infowindow_open : [0],
					infowindow_content : results[0].formatted_address,
					lat : results[0].geometry.location.lat(),
					lng : results[0].geometry.location.lng(),
					name : results[0].formatted_address
				};
				//create new post for new location
				var data = {
					data : {
						action : 'quick_add_location_to_map',
						location : geocode_location
					},
					type : 'post',
					dataType : 'json',
					url : ajaxurl,
					success : function(response){
						if(response.status == 1){
							geocode_location.id = parseInt(response.id);
							instance.place_marker(geocode_location, true);
							instance.fit_bounds();
							$('<input type="checkbox" checked="checked" class="map-locations-checkbox" id="'+geocode_location.id+'" name="map_locations[location_ids]['+geocode_location.id+']" value="1">&nbsp;<label class="for-checkbox" for="'+geocode_location.id+'">'+geocode_location.name+'</label><br />').prependTo('.map-location-checkbox-container');
						}
						else
							alert(response.message);
					},
					error : function(response){},
					complete : function(){callback();}
				};
				$.ajax(data);
			}
			else
				alert(L10.unsuccessful+": "+status);
		});
	};
	ResponsiveGoogleMap.prototype.update_map_type_preview = function(event, ui, picker){
		var output_styles = new Array();
		if(event.type == 'irischange')
			picker.val(ui.color.toString());

		$('.custom-map-type-input').each(function(){
			var value;
			if($(this).is(':radio')){
				if($(this).is(':checked')){
					var features = $(this).attr('id');
					features = features.substr(0,(features.length-1));
					value = $(this).val();
				}
				else
					return;
			}
			else if($(this).is('select')){
				var features = $(this).attr('id');
				value = $('option:selected', this).val();
			}
			else {
				var features = $(this).attr('id');
				value = $(this).val();
			}
			features = features.split('|');
			if(features.length >= 3){
				var style = {
					featureType: features[0],
					elementType: features[1],
					stylers : [	{}]
				};
				style.stylers[0][features[2]] = $(this).val();
				
				if($('input[name="custom_map_type[enable_changes_to]['+features[0]+']"]').is(':checked')){
					var inserted = false;
					for(var i=0;i<output_styles.length;i++){
						if(output_styles[i]['featureType'] == style['featureType'] &&
						output_styles[i]['elementType'] == style['elementType']){
							inserted = true;
							var length = output_styles[i]['stylers'].length;
							output_styles[i]['stylers'].push({});
							output_styles[i]['stylers'][length][features[2]] = $(this).val();
							break;
						}
					}
					if(!inserted)
						output_styles.push(style);
				}
			}
		});
		this.map.setOptions({'styles': output_styles});
	};
	ResponsiveGoogleMap.prototype.bind_events = function(){
		bind_events.call(this);
		var instance = this;
		if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.update_default_center === '1'){
			var zoom_listener = google.maps.event.addListener(this.map, 'center_changed', function(){
				$('#center_lat').val(instance.map.getCenter().lat());
				$('#center_lng').val(instance.map.getCenter().lng());
			});
			var pan_listener = google.maps.event.addListener(this.map, 'zoom_changed', function(){
				$('#zoom_level').val(instance.map.getZoom()).attr('selected',true);
			});
		}
		$(document).on('click', '.geocode_address_button', function(event){
			event.preventDefault();
			instance.geocode_single($('.address_to_geocode').val(), function(){});
		});
		$(document).on('click', '.geocode_add_address_button', function(event){
			event.preventDefault();
			instance.quick_add_geocode_address($('.address_to_geocode').val(), function(){$('.address_to_geocode').val('').focus();});
		});
		$(document).on('keydown', function(event){
			if($('.address_to_geocode').is(":focus") && event.keyCode == 13){
				event.preventDefault();
				instance.quick_add_geocode_address($('.address_to_geocode').val(), function(){$('.address_to_geocode').val('').focus();});
				return false;
			}
		});
		if(typeof(ajaxurl) != 'undefined'){
			$(document).on('change', '.map-locations-checkbox', function(event){
				var id = this.id;
				var elm = $(this)
				if($(this).is(':checked')){
					var data = {
						data : {
							action : 'add_location_to_map',
							location_id : id
						},
						type : 'post',
						dataType : 'json',
						url : ajaxurl,
						success : function(response){
							if(response.status == 1){
								instance.place_marker(response.location, true);
								instance.fit_bounds();
								elm.attr('checked',true);
							}
							else
								alert(response.message);
						},
						error : function(response){},
						complete : function(){}
					};
					$.ajax(data);
				}
				else {
					instance.remove_marker_by_id(parseInt(id), true);
					if(instance.marker_array.length)
						instance.fit_bounds();
				}
			});
		}
		// bind preview events
		if(typeof pagenow !== 'undefined' && pagenow == 'custom-map-types'){
			instance.map.setOptions({ 'styles': CS });
			if($('#enable_custom_json0').is(':checked')){
				$(document).on('change keyup input irischange', '.custom-map-type-input', function(event, ui){
					instance.update_map_type_preview(event, ui, $(this));
				});
				$(document).on('click', '.wp-picker-clear', function(event){
					instance.update_map_type_preview(event);
				});
			}
		}
	};
}(ResponsiveGoogleMap.prototype.bind_events, jQuery, custom_style = typeof custom_style !== 'undefined' ? custom_style : null, RGMLocal = typeof RGMLocal !== 'undefined' ? RGMLocal : null));