//compatibility for IE with Object.create
if (!Object.create || typeof Object.create !== 'function') { Object.create = function(o) { function F(){}; F.prototype = o; return new F(); } }
var ModuleSettings = function(settings){
	var hasOwnProperty = function(prop){
		var proto = this.__proto__ || this.constructor.prototype;
		return (prop in this) && (!(prop in proto) || proto[prop] !== this[prop]);
	};
	var deepCopy = function(p,c){
		var c = c||{};
		for(var i in p){
			if(typeof p[i] === 'object'){
				c[i] = (p[i].constructor === Array)?[]:{};
				deepCopy(p[i],c[i]);
			}
			else c[i] = p[i];
		}
		return c;
	};
	this.getSetting = function(property){
		if(module_settings.hasOwnProperty(property)){
			if(Object.prototype.toString.call(module_settings[property]) === '[object Array]'){
				if(module_settings[property].length == 1)
					return module_settings[property][0];
				else
					return module_settings[property];
			}
			else
				return module_settings[property];
		}
		else
			return false;
	};
	this.setSetting = function(property, value){
		if(Object.prototype.toString.call(module_settings[property]) === '[object Object]') 
			module_settings[property] = Object(value);
		else if(Object.prototype.toString.call(module_settings[property]) === '[object Array]'){
			if(Object.prototype.toString.call(value) === '[object Array]')
				module_settings[property] = value;
			else
				module_settings[property] = Array(value);
		}
		else if(Object.prototype.toString.call(module_settings[property]) === '[object Number]') 
			module_settings[property] = Number(value);
		else if(Object.prototype.toString.call(module_settings[property]) === '[object Boolean]'){
			if(value === 'false')
				module_settings[property] = false;
			else
				module_settings[property] = Boolean(value);
		}
		else
			module_settings[property] = value;
	};
	this.setOptions = function(new_module_settings){
		for(var property in new_module_settings){
			this.setSetting(property, new_module_settings[property]);
		}
	}
	//copy the values to private variable
	var module_settings = deepCopy(settings);
	return {
		getSetting: this.getSetting,
		setSetting: this.setSetting,
		setOptions: this.setOptions
    };
};

var ResponsiveGoogleMap = (function($, GS, MS, L10n)
{
	var ResponsiveGoogleMap = function(canvas_id, locations, settings){
		// Return if jQuery or Global settings not available
		if($ === null || GS === null || L10n === null || typeof canvas_id === 'undefined')
			return;

		ModuleSettings.call(this, GS);
		this.map_canvas = document.getElementById(canvas_id);
		this.canvas_id = canvas_id;
		this.directions_container = '#'+canvas_id+'_directions_container';
		this.directions_form = '#'+canvas_id+'_directions_form';
		this.map_container = '#'+canvas_id+'_container';
		this._fullscreen = false;
		this._resize_timer = false;
		this.scroll_pos = {x:window.scrollX,y:window.scrollY};

		// Load the Google Maps API if not loaded or loading
		if(!ResponsiveGoogleMap.api_loading && !ResponsiveGoogleMap.api_loaded){
			ResponsiveGoogleMap.api_loading = true;
			var script = document.createElement("script");
			script.type = "text/javascript";
			if(this.getSetting('map_api_key'))
				script.src = "https://maps.google.com/maps/api/js?key="+this.getSetting('map_api_key')+"&sensor=true&libraries=weather,panoramio,adsense&callback=ResponsiveGoogleMap.api_loaded_callback";
			else
				script.src = "https://maps.google.com/maps/api/js?sensor=true&libraries=weather,panoramio,adsense&callback=ResponsiveGoogleMap.api_loaded_callback";
			document.body.appendChild(script);
		}
		if(MS !== null && MS.length > 0 && !ResponsiveGoogleMap.styles_loading && !ResponsiveGoogleMap.styles_loaded)
			this.load_styles();
		else if(MS == null || MS.length == 0)
			ResponsiveGoogleMap.styles_loaded = true;

		if(typeof settings !== 'undefined')
			this.setOptions(settings);
		
		locations = typeof locations !== 'undefined' ? locations : null;
		if(locations != null && typeof locations.async !== 'undefined' && locations.async == 1)
			this.location_ids = locations.locations;
		else
			this.locations = locations;
		
		this.init();
	};
	
	ResponsiveGoogleMap.prototype = Object.create(ModuleSettings.prototype);
	
	ResponsiveGoogleMap.prototype.constructor = ResponsiveGoogleMap;
	
	ResponsiveGoogleMap.api_loaded = false;
	ResponsiveGoogleMap.api_loading = false;
	ResponsiveGoogleMap.api_loaded_callback = function(){
		ResponsiveGoogleMap.api_loaded = true;
		ResponsiveGoogleMap.api_loading = false;
	};
	
	ResponsiveGoogleMap.styles_loaded = false;
	ResponsiveGoogleMap.styles_loading = false;
	ResponsiveGoogleMap.styles_loaded_callback = function(){
		ResponsiveGoogleMap.styles_loaded = true;
		ResponsiveGoogleMap.styles_loading = false;
	};
	
	ResponsiveGoogleMap.prototype.init = function(){
		var instance = this;
		if(ResponsiveGoogleMap.api_loaded && ResponsiveGoogleMap.styles_loaded){
			this.continue_init();
		}
		else {
			this.loading_timer = setInterval(function(){
				if(ResponsiveGoogleMap.api_loaded && ResponsiveGoogleMap.styles_loaded){
					instance.continue_init();
					clearInterval(instance.loading_timer);
					delete instance.loading_timer;
					return;
				}
			}, 100);
		}
	};
	
	ResponsiveGoogleMap.prototype.load_styles = function(styles){
		ResponsiveGoogleMap.styles_loading = true;
		var instance = this;
		var data = {
			data : {
				action : 'get_available_styles',
				styles : MS
			},
			dataType : 'json',
			type : Boolean(parseInt(this.getSetting('enable_front_ajax_cache'))) ? 'get' : 'post',
			cache : Boolean(parseInt(this.getSetting('enable_front_ajax_cache'))),
			url : GS.ajaxurl,
			success : function(response){
				if(response.status == 1){
					MS = response.styles;
					ResponsiveGoogleMap.styles_loaded_callback();
				}
			},
			error : function(response){},
			complete : function(response){}
		};
		$.ajax(data);
	};
	
	ResponsiveGoogleMap.prototype.continue_init = function(){
		if(this.map_canvas != null){
			var has_extra_maps = false;
			var mapTypeIds = [google.maps.MapTypeId.ROADMAP, google.maps.MapTypeId.SATELLITE, google.maps.MapTypeId.HYBRID, google.maps.MapTypeId.TERRAIN];
			var enabled_maps;
			if(this.getSetting('enable_map_types') !== false && MS !== null){
				enabled_maps = this.getSetting('enable_map_types');
				if(Object.prototype.toString.call(enabled_maps) === '[object Object]'){
					var temp_enabled_maps = [];
					for(i in enabled_maps)	{
						temp_enabled_maps.push(i);
					}
					enabled_maps = temp_enabled_maps;
				}
				else if(Object.prototype.toString.call(enabled_maps) !== '[object Array]') 
					enabled_maps = Array(enabled_maps.toString());
				
				for(var i=0;i<enabled_maps.length;i++){
					has_extra_maps = true;
					if(enabled_maps[i] in MS)
						mapTypeIds.push(MS[enabled_maps[i]].id);
				}
			}
			
			var mapTypeId = Number(this.getSetting('map_type'))
			if(isNaN(mapTypeId))
				mapTypeId = google.maps.MapTypeId[this.getSetting('map_type').toUpperCase()];
			else if(MS != null && mapTypeId in MS) {
				enabled_maps.push(parseInt(mapTypeId));
				mapTypeId = mapTypeId.toString();
				mapTypeIds.push(MS[mapTypeId].id);
				has_extra_maps = true;
			}
			else
				mapTypeId = google.maps.MapTypeId.ROADMAP;

			var map_type_control_style = null;
			if(Boolean(parseInt(this.getSetting('enable_map_type_control_dropdown'))))
				map_type_control_style = google.maps.MapTypeControlStyle.DROPDOWN_MENU;
			
			this.is_basic = false
			this.marker_array = [];
			this.infowindow_array = [];
			this.directions_array = [];
			this.old_directions = [];
			this.directions_display = null;
			this.map_options = {
				center: new google.maps.LatLng(this.getSetting('center_lat'), this.getSetting('center_lng')),
				zoom: parseInt(this.getSetting('zoom_level')),
				overviewMapControl: Boolean(parseInt(this.getSetting('show_overview_map_control'))),
				overviewMapControlOptions: {opened: Boolean(parseInt(this.getSetting('overview_map_control_opened')))},
				tilt: parseInt(this.getSetting('tilt')),
				scrollwheel: Boolean(parseInt(this.getSetting('enable_scroll_wheel_zoom'))),
				zoomControl: Boolean(parseInt(this.getSetting('enable_zoom_control'))),
				scaleControl: Boolean(parseInt(this.getSetting('enable_scale_control'))),
				rotateControl: Boolean(parseInt(this.getSetting('enable_rotate_control'))),
				panControl: Boolean(parseInt(this.getSetting('enable_pan_control'))),
				mapTypeControl: Boolean(parseInt(this.getSetting('enable_map_type_control'))),
				streetViewControl: Boolean(parseInt(this.getSetting('enable_street_view_control'))),
				disableDoubleClickZoom: Boolean(parseInt(this.getSetting('disable_doubleclick_zoom'))),
				mapTypeId: mapTypeId,
				backgroundColor: this.getSetting('map_bg_color'),
				mapTypeControlOptions: {
				  mapTypeIds: mapTypeIds,
				  style: map_type_control_style
				}
			};
			this.geocoder = new google.maps.Geocoder();
			this.info_window_max_width = parseInt(this.getSetting('info_window_max_width'));
			if(!this.info_window_max_width)
				this.info_window_max_width = 350;
			
			if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.location_page === "1"){
				this.is_basic = true;
				this.map_options['panControl'] = false;
				this.map_options['streetViewControl'] = false;
				this.map_options['overviewMapControl'] = false;
			}
			
			if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.directions)
				this.setSetting('enable_directions', map_page_admin_settings.directions);
			
			if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.autocenter)
				this.setSetting('autocenter', map_page_admin_settings.autocenter);
				
			if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.show_user)
				this.setSetting('show_user_location', map_page_admin_settings.show_user);
			
			if(typeof map_page_admin_settings !== 'undefined' && map_page_admin_settings.draggable)
				this.draggable = Boolean(parseInt(map_page_admin_settings.draggable));
			else
				this.draggable = false;
			
			this.map = new google.maps.Map(this.map_canvas,this.map_options);
			
			if(Boolean(parseInt(this.getSetting('enable_weather_layer')))){
				var temp = this.getSetting('weather_layer_temp');
				if(temp)
					temp = temp.toUpperCase();
				else
					temp = 'FAHRENHEIT';

				var wind_speed = this.getSetting('weather_layer_wind_speed');
				if(wind_speed)
					wind_speed = wind_speed.replace(/ /g,"_").toUpperCase();
				else
					wind_speed = 'MILES_PER_HOUR';
					
				var weatherLayer = new google.maps.weather.WeatherLayer({
					temperatureUnits: google.maps.weather.TemperatureUnit[temp],
					windSpeedUnits: google.maps.weather.WindSpeedUnit[wind_speed]
				});
				weatherLayer.setMap(this.map);
			}
			
			if(Boolean(parseInt(this.getSetting('enable_cloud_layer'))))
				new google.maps.weather.CloudLayer().setMap(this.map);
			
			if(Boolean(parseInt(this.getSetting('enable_bicycle_layer'))))
				new google.maps.BicyclingLayer().setMap(this.map);
				
			if(Boolean(parseInt(this.getSetting('enable_transit_layer'))))
				new google.maps.TransitLayer().setMap(this.map);
				
			if(Boolean(parseInt(this.getSetting('enable_traffic_layer'))))
				new google.maps.TrafficLayer().setMap(this.map);
			
			if(has_extra_maps){
				for(var i=0;i<enabled_maps.length;i++){
					if(enabled_maps[i] in MS){
						var customMapType = new google.maps.StyledMapType(MS[enabled_maps[i]].style, {name : MS[enabled_maps[i]].name});
						this.map.mapTypes.set(MS[enabled_maps[i]].id, customMapType);
					}
				}
			}
			
			if(Boolean(parseInt(this.getSetting('enable_panoramio_layer')))){
				var panoramioLayer = new google.maps.panoramio.PanoramioLayer();
				if(this.getSetting('panoramio_filter_id'))	
					panoramioLayer.setUserId(this.getSetting('panoramio_filter_id'));
				if(this.getSetting('panoramio_filter_tag'))
					panoramioLayer.setTag(this.getSetting('panoramio_filter_tag'));
				panoramioLayer.setMap(this.map);
			}
  
			// Add custom nav color
			this.load_nav_styles();			
			if(Boolean(parseInt(this.getSetting('enable_map_nav_color'))) ||
				Boolean(parseInt(this.getSetting('enable_fullscreen_view'))) ||
				Boolean(parseInt(this.getSetting('enable_map_reset_button'))) ||
				Boolean(parseInt(this.getSetting('enable_autofit_button'))))
			this.add_legend();
			this.add_map_buttons();
			this.add_close_button();
			this.add_directions_container();
			setup_directions(this);
			this.load_map();
			this.bind_events();
		}
	};
	
	ResponsiveGoogleMap.prototype.load_nav_styles = function()	{
		var css;
		var hex_color = typeof this.getSetting('map_nav_color') === 'string' ? this.getSetting('map_nav_color') : '#000000';
		var color = hex_to_hrgb(hex_color);
		if(!color && Boolean(parseInt(this.getSetting('enable_map_nav_color'))))
			return;
			
		if(!Boolean(parseInt(this.getSetting('enable_map_nav_color'))))
			css = '#'+this.canvas_id+' .map-nav-bg {background:none;}';
		else
			css = '#'+this.canvas_id+' .map-nav-bg {'+
				'background-color: rgba('+color.r+', '+color.g+', '+color.b+', 0.5);'+
				'filter:progid:DXImageTransform.Microsoft.gradient(startColorstr=#7f'+color.h+', endColorstr=#7f'+color.h+');'+
				'-ms-filter: "progid:DXImageTransform.Microsoft.gradient(startColorstr=#7f'+color.h+', endColorstr=#7f'+color.h+')";'+
				'}';
		
		var head = document.getElementsByTagName('head')[0];
		var style = document.createElement('style');
		style.type = 'text/css';
		if(style.styleSheet)
			style.styleSheet.cssText = css;
		else
			style.appendChild(document.createTextNode(css));
			
		head.appendChild(style);
	};
	
	// load the map and place all the markers
	// can be used to reset the map to original
	ResponsiveGoogleMap.prototype.load_map = function(){
		var instance = this;
		this.user_location = null;	
		this.clear_directions();
		this.remove_all_markers();
		this.infowindows_to_open = new Array();
		if(this.markerClusterer)
			this.markerClusterer.clearMarkers();
		this.markerClusterer = null;
		this.map.setCenter(this.map_options.center);
		this.map.setZoom(this.map_options.zoom);
		if(Boolean(parseInt(this.getSetting('enable_directions'))) && Boolean(parseInt(this.getSetting('directions_open_by_default'))))
			this.toggle_directions_container('open');
		else
			this.toggle_directions_container('close');
		this.load_locations();
	};
	
	ResponsiveGoogleMap.prototype.display_locations = function(locations){
		marker_locations = typeof locations !== 'undefined' ? locations : this.locations;
		if(marker_locations)
			this.place_markers(marker_locations);
		var instance = this;	
		if(navigator.geolocation && Boolean(parseInt(this.getSetting('show_user_location')))){
			var geolocation_options = {
				timeout : 5000,
				enableHighAccuracy: true,
				maximumAge: 0
			};
			navigator.geolocation.getCurrentPosition(function(pos){
				instance.user_location = {
					geocode_address : pos.coords.latitude+', '+pos.coords.longitude,
					infowindow_open : ["0"],
					image : new google.maps.MarkerImage('//maps.gstatic.com/mapfiles/mobile/mobileimgs2.png',
                                                    new google.maps.Size(22,22),
                                                    new google.maps.Point(0,18),
                                                    new google.maps.Point(11,11)),
					infowindow_content : "<h2>"+L10n.you_are_here+"<\/h2>\n<p>Lat: "+pos.coords.latitude+"<br />Lng: "+pos.coords.longitude+"</p>",
					lat : pos.coords.latitude,
					lng : pos.coords.longitude,
					id : 'you_are_here',
					name : L10n.you
				};
				instance.place_marker(instance.user_location);
				if(Boolean(parseInt(instance.getSetting('autocenter'))))
					instance.fit_bounds();
				if(Boolean(parseInt(instance.getSetting('enable_directions')))){
					instance.direction_input_a.placeholder = L10n.my_location;
					instance.direction_input_b.placeholder = L10n.my_location;
				}
			}, function(){}, geolocation_options);
		}
		if(Boolean(parseInt(this.getSetting('enable_marker_clustering')))){
			var cluster_images = this.getSetting('cluster_images');
			if(cluster_images){
				if(Object.prototype.toString.call(cluster_images) === '[object Object]')
					cluster_images = Array(cluster_images);
				this.markerClusterer = new MarkerClusterer(this.map, this.marker_array, {styles:cluster_images});
			} else {
				this.markerClusterer = new MarkerClusterer(this.map, this.marker_array, {});
			}
		}
		google.maps.event.addListenerOnce(this.map, 'idle', function(){
			for(i in instance.infowindows_to_open){
				var marker = instance.get_marker_by_id(instance.infowindows_to_open[i]);
				var infowindow = instance.get_infowindow_by_id(instance.infowindows_to_open[i]);
				infowindow.open(instance.map,marker);
			}
		});
		if(Boolean(parseInt(this.getSetting('autocenter'))) && this.marker_array.length)
			this.fit_bounds();
	};
	
	ResponsiveGoogleMap.prototype.load_locations = function(){
		if(typeof this.locations !== 'undefined')
			this.display_locations();
		else {
			var instance = this;
			var data = {
				data : {
					action : 'get_locations_by_ids',
					ids : this.location_ids
				},
				type : Boolean(parseInt(this.getSetting('enable_front_ajax_cache'))) ? 'get' : 'post',
				cache : Boolean(parseInt(this.getSetting('enable_front_ajax_cache'))),
				dataType : 'json',
				url : GS.ajaxurl,
				success : function(response){
					if(response.status == 1){
						if(typeof instance.locations === 'undefined')
							instance.locations = response.locations;
						instance.display_locations();
					}
				},
				error : function(response){},
				complete : function(response){}
			};
			$.ajax(data);
		}
	};
	
	ResponsiveGoogleMap.prototype.clear_directions = function(){
		for(var i=0;i<this.directions_array.length;i++)
			this.directions_array[i].setMap(null);
		this.directions_array = new Array();	
		this.current_directions = null;
		this.directions_service = new google.maps.DirectionsService();
		this.old_directions = new Array();
		
		if(this.directions_display != null)
			this.directions_display.set('directions', null);
		this.directions_response.innerHTML = '';
		this.directions_response_div.innerHTML = '';
		this.direction_input_a.value = '';
		this.direction_input_b.value = '';
		
		if(this._fullscreen)
			this.directions_display.setPanel(this.directions_response);
		else
			this.directions_display.setPanel(this.directions_response_div);
		this.set_reverse_directions();
		this.set_undo_disabled(true);
		this.bounds = new google.maps.LatLngBounds();
		for(var i=0;i<this.marker_array.length;i++)
			this.bounds.extend(this.marker_array[i].position);
		if(this.marker_array.length)
			this.fit_bounds();
	}
	
	ResponsiveGoogleMap.prototype.get_marker_by_id = function(id){
		for(var i=0; i<this.marker_array.length; i++){
			if(this.marker_array[i]['id'] === id)
				return this.marker_array[i];
		}
	};
	
	ResponsiveGoogleMap.prototype.get_infowindow_by_id = function(id){
		for(var i=0; i<this.infowindow_array.length; i++){
			if(this.infowindow_array[i]['id'] === id)
				return this.infowindow_array[i];
		}
	};
	
	ResponsiveGoogleMap.prototype.remove_marker_by_id = function(id, remove_from_defaults){
		remove_from_defaults = typeof remove_from_defaults !== 'undefined' ? remove_from_defaults : false;
		if(remove_from_defaults){
			for(var i=0;i<this.locations.length;i++){
				if(this.locations[i]['id'] === id){
					this.locations.splice(i,1);
					break;
				}
			}
		}
		for(var i=0;i<this.marker_array.length;i++){
			if(this.marker_array[i]['id'] === id)	{
				this.marker_array[i].setMap(null);
				this.marker_array.splice(i,1);
				break;
			}
		}
		for(var i=0;i<this.infowindow_array.length;i++){
			if(this.infowindow_array[i]['id'] === id){
				this.infowindow_array[i].setMap(null);
				this.infowindow_array.splice(i,1);
				break;
			}
		}
		this.bounds = new google.maps.LatLngBounds();
		for(var i=0;i<this.marker_array.length;i++)
			this.bounds.extend(this.marker_array[i].position);
	};
	
	ResponsiveGoogleMap.prototype.remove_all_markers = function(){
		for(var i=0;i<this.marker_array.length;i++)
			this.marker_array[i].setMap(null);
		for(var i=0; i<this.infowindow_array.length; i++)
			this.infowindow_array[i].setMap(null);
		this.marker_array = new Array();
		this.infowindow_array = new Array();
		this.bounds = new google.maps.LatLngBounds();
	};
	
	// used to place a group of marker on the map
	ResponsiveGoogleMap.prototype.place_markers = function(locations){
		if(Object.prototype.toString.call( locations ) === '[object Array]'){
			for (var i = 0; i < locations.length; i++) 
				this.place_marker(locations[i]);
		}
		else
			this.place_marker(locations);
	};
	
	ResponsiveGoogleMap.prototype.place_marker = function(location, add_to_defaults){
		if(location == null) return;
		add_to_defaults = typeof add_to_defaults !== 'undefined' ? add_to_defaults : false;
		if(add_to_defaults)
			this.locations.push(location);
			
		var image;
		var id;
		var instance = this;
		
		if(location.image)
			image = location.image;
		if(location.id)
			id = location.id;
		else
			id = ResponsiveGoogleMap.generate_uid();
		
		var marker = new google.maps.Marker({
			map:this.map,
			draggable:this.draggable,
			icon: image,
			title: location.name,
			animation: google.maps.Animation.DROP,
			position: new google.maps.LatLng(location.lat, location.lng),
			id : id
		});
		
		var infowindow = new google.maps.InfoWindow({
			maxWidth: this.info_window_max_width,
			content: this.get_infowindow_content(location),
			id : id
		});

		this.infowindow_array.push(infowindow);
		
		google.maps.event.addListener(marker, 'click', function(){
			instance.close_all_infowindows();
			infowindow.open(instance.map,marker);
		});
		google.maps.event.addListener(marker, 'dragend', function(){
			instance.close_all_infowindows();
			ResponsiveGoogleMap.update_position(marker);
		});
		google.maps.event.addListener(infowindow,'domready',function(){
			//fix for content scroll
			this.getContent().parentNode.style.overflowY = 'auto';
			this.getContent().parentNode.style.overflowX = 'hidden';
		});
		if(Boolean(parseInt(location.infowindow_open)))
			this.infowindows_to_open.push(location.id);
		
		this.bounds.extend(marker.position);
		this.marker_array.push(marker);
	};
	
	ResponsiveGoogleMap.prototype.add_legend = function(){
		this.legend_div = document.createElement('div');
		this.legend_div.className = "legend-container map-nav-bg";
		this.legend_div.index = 1;
		this.map.controls[google.maps.ControlPosition.TOP_CENTER].push(this.legend_div);
	};
	
	ResponsiveGoogleMap.prototype.fit_bounds = function(force_fit){
		force_fit = typeof force_fit !== 'undefined' ? force_fit : false;
		if((typeof this.locations == 'undefined' || (Object.prototype.toString.call(this.locations) === '[object Array]' && !this.locations.length) && !this.is_basic))
			return;
		if(this.bounds.getNorthEast().equals(this.bounds.getSouthWest())){
		   var extendPoint1 = new google.maps.LatLng(this.bounds.getNorthEast().lat() + 0.0005, this.bounds.getNorthEast().lng() + 0.0005);
		   var extendPoint2 = new google.maps.LatLng(this.bounds.getNorthEast().lat() - 0.0005, this.bounds.getNorthEast().lng() - 0.0005);
		   this.bounds.extend(extendPoint1);
		   this.bounds.extend(extendPoint2);
		}
		this.map.fitBounds(this.bounds);
	};
	
	ResponsiveGoogleMap.prototype.add_map_buttons = function(){
		var display_menu = false;
		this.map_button_dropdown = document.createElement('ul');
		this.map_button_dropdown.className = 'map-button-dropdown';
		this.map_button_dropdown.style.display = 'none';
		var li = document.createElement('li');
		li.innerHTML = '<div class="legend-button">'+L10n.options+'<div class="legend-button-down-arrow"></div></div>';
		var ul = document.createElement('ul');
		
		if(Boolean(parseInt(this.getSetting('enable_directions')))){
			var temp_li = document.createElement('li');
			temp_li.appendChild(this.add_directions_button());
			ul.appendChild(temp_li);
			display_menu = true;
		}
			
		if(!this.is_basic){
			if(Boolean(parseInt(this.getSetting('enable_map_reset_button')))){
				var temp_li = document.createElement('li');
				temp_li.appendChild(this.add_reset_button());
				ul.appendChild(temp_li);
				display_menu = true;
			}
			if(Boolean(parseInt(this.getSetting('enable_autofit_button')))){
				var temp_li = document.createElement('li');
				temp_li.appendChild(this.add_autocenter_button());
				ul.appendChild(temp_li);
				display_menu = true;
			}
		}
		
		if(Boolean(parseInt(this.getSetting('enable_fullscreen_view'))) && display_menu){
			var temp_li = document.createElement('li');
			temp_li.appendChild(this.add_fullscreen_button());
			ul.insertBefore(temp_li,ul.firstChild);
			
			var temp_li = document.createElement('li');
			temp_li.appendChild(this.add_exit_fullscreen_button());
			ul.insertBefore(temp_li,ul.firstChild);
		}
		
		li.appendChild(ul);
		this.map_button_dropdown.appendChild(li);
		this.map_button_dropdown.index = 1;

		if(Boolean(parseInt(this.getSetting('enable_map_options_dropdown'))) && display_menu)
			this.map_button_dropdown.style.display = 'block';

		if(!Boolean(parseInt(this.getSetting('enable_map_options_dropdown')))){
			if(!this.is_basic){
				if(Boolean(parseInt(this.getSetting('enable_map_reset_button'))))
					this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(this.add_reset_button());
				if(Boolean(parseInt(this.getSetting('enable_autofit_button'))))
					this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(this.add_autocenter_button());
			}
		}
		
		if(display_menu)
			this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(this.map_button_dropdown);
		if(Boolean(parseInt(this.getSetting('enable_fullscreen_view'))))
			this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(this.add_fullscreen_icon());
	};
	
	ResponsiveGoogleMap.prototype.adjust_map_dropdown = function(event){
		var drop_down = $(this.map_container+' img[src="https://maps.gstatic.com/mapfiles/arrow-down.png"]').parent();
		drop_down.parent().width('auto');
		var menu_width = drop_down.siblings('div').show().width();
		drop_down.parent().width(menu_width + 25);
		drop_down.siblings('div').hide();
		google.maps.event.trigger(this.map, "resize");
	};
			
	ResponsiveGoogleMap.prototype.add_autocenter_button = function(){
		this.center_text = document.createElement('div');
		this.center_text.innerHTML = L10n.auto_fit_map;
		this.center_text.title = L10n.auto_fit_map;
		this.center_text.className = "legend-button autocenter-button";

		var instance = this;
		google.maps.event.addDomListener(this.center_text, 'click', function(){
			if(instance.marker_array.length)
				instance.fit_bounds();
			else
				instance.load_map();
		});
		
		this.center_text.index = 2;
		return this.center_text;
	};
	
	ResponsiveGoogleMap.prototype.add_reset_button = function(){
		this.reset_text = document.createElement('div');
		this.reset_text.innerHTML = L10n.reset_map;
		this.reset_text.title = L10n.reset_map;
		this.reset_text.className = "legend-button reset-button";

		var instance = this;
		google.maps.event.addDomListener(this.reset_text, 'click', function(){
			instance.load_map();
		});
		
		this.reset_text.index = 2;
		return this.reset_text;
	};
	
	ResponsiveGoogleMap.prototype.add_directions_button = function(){
		this.get_directions_button = document.createElement('div');
		this.get_directions_button.innerHTML = L10n.get_directions;
		this.get_directions_button.title = L10n.get_directions;
		this.get_directions_button.className = "legend-button directions-button";

		var instance = this;
		google.maps.event.addDomListener(this.get_directions_button, 'click', function(){
			instance.toggle_directions_container('open');
		});
		
		this.get_directions_button.index = 1;
		return this.get_directions_button;
	};
	
	ResponsiveGoogleMap.prototype.add_fullscreen_button = function(){
		this.fullscreen_text = document.createElement('div');
		this.fullscreen_text.innerHTML = L10n.full_screen;
		this.fullscreen_text.title = L10n.full_screen;
		this.fullscreen_text.className = "legend-button fullscreen-button";
		
		if(window.innerWidth<=400)
			this.fullscreen_text.style.display = 'none';
		else if(!this._fullscreen)
			this.fullscreen_text.style.display = 'block';

		var instance = this;
		google.maps.event.addDomListener(this.fullscreen_text, 'click', function(){
			instance.goto_large_map();
		});
		
		this.fullscreen_text.index = 1;
		return this.fullscreen_text;
	};
	
	ResponsiveGoogleMap.prototype.add_exit_fullscreen_button = function(){
		this.exit_fullscreen_text = document.createElement('div');
		this.exit_fullscreen_text.innerHTML = L10n.exit_full_screen;
		this.exit_fullscreen_text.title = L10n.exit_full_screen;
		this.exit_fullscreen_text.className = "legend-button fullscreen-exit-button";
		this.exit_fullscreen_text.style.display = 'none';

		var instance = this;
		google.maps.event.addDomListener(this.exit_fullscreen_text, 'click', function(){
			instance.goto_small_map();
		});
		
		this.exit_fullscreen_text.index = 1;
		return this.exit_fullscreen_text;
	};
	
	ResponsiveGoogleMap.prototype.add_fullscreen_icon = function(){
		this.fullscreen_icon = document.createElement('div');
		this.fullscreen_icon.title = L10n.full_screen;
		this.fullscreen_icon.innerHTML = '&nbsp;';
		this.fullscreen_icon.className = "legend-button fullscreen-icon";
		
		if(window.innerWidth<=400)
			this.fullscreen_icon.style.display = 'none';
		else if(!this._fullscreen)
			this.fullscreen_icon.style.display = 'block';

		var instance = this;
		google.maps.event.addDomListener(this.fullscreen_icon, 'click', function(){
			instance.goto_large_map();
		});
		
		this.fullscreen_icon.index = -100;
		return this.fullscreen_icon;
	};
	
	ResponsiveGoogleMap.prototype.add_close_button = function(){
		this.close_text = document.createElement('div');
		this.close_text.title = L10n.exit_full_screen;
		this.close_text.className = "legend-button close-button";
		this.close_text.innerHTML = 'X';
		this.close_text.style.display = 'none';

		var instance = this;
		google.maps.event.addDomListener(this.close_text, 'click', function(){
			instance.goto_small_map();
		});
		
		this.close_text.index = 0;
		this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(this.close_text);
	};
	
	ResponsiveGoogleMap.prototype.add_directions_container = function(){
		var instance = this;
		var spacer = document.createElement('div');
		spacer.style.width = '1px';
		spacer.style.height = '30px';
		spacer.style.padding = '0px';
		spacer.style.margin = '0px';
		spacer.index = -1;
		this.map.controls[google.maps.ControlPosition.TOP_RIGHT].push(spacer);

		this.directions_div_status = 'closed';
		this.directions_div = document.createElement('div');
		this.directions_div.className = "direction-div";
		this.directions_div.style.width = "0px";
		this.directions_div.style.display = 'none';
		
		this.directions_grabber = document.createElement('div');
		this.directions_grabber.className = "direction-grabber map-nav-bg";
		this.directions_grabber.innerHTML = "<span>&lt;</span>";
		
		this.directions_div.appendChild(this.directions_grabber);
		
		this.directions_ui = document.createElement('div');
		this.directions_ui.className = "direction-container";
		this.directions_div.appendChild(this.directions_ui);
		
		this.driection_form = document.createElement('form');
		this.driection_form.setAttribute('method', 'post');
		this.driection_form.setAttribute('action', '#');
		this.directions_ui.appendChild(this.driection_form);
		$(this.driection_form).on('submit', function(event){
			instance.cancel_default_action(event);
			instance.calc_route();
			return false;
		});
		
		// A input
		var input_wrap_div_a = document.createElement('div');
		input_wrap_div_a.className = 'input-wrap';
		
		var input_div_a = document.createElement('div');
		input_div_a.className = 'dir-input';
		input_wrap_div_a.appendChild(input_div_a);
		
		var input_div_a_letter = document.createElement('div');
		input_div_a_letter.className = 'kd-button-submit input-letter letter-a';
		input_div_a_letter.innerHTML = L10n.a;
		input_div_a.appendChild(input_div_a_letter);
		
		this.direction_input_a = document.createElement('input');
		this.direction_input_a.setAttribute('type', 'text');
		this.direction_input_a.className = 'wp kd-input-text';
		input_div_a.appendChild(this.direction_input_a);
		
		this.driection_form.appendChild(input_wrap_div_a);
		
		// B input
		var input_wrap_div_b = document.createElement('div');
		input_wrap_div_b.className = 'input-wrap';
		
		var input_div_b = document.createElement('div');
		input_div_b.className = 'dir-input';
		input_wrap_div_b.appendChild(input_div_b);
		
		var input_div_b_letter = document.createElement('div');
		input_div_b_letter.className = 'kd-button-submit input-letter letter-b';
		input_div_b_letter.innerHTML = L10n.b;
		input_div_b.appendChild(input_div_b_letter);
		
		this.direction_input_b = document.createElement('input');
		this.direction_input_b.setAttribute('type', 'text');
		this.direction_input_b.className = 'wp kd-input-text';
		input_div_b.appendChild(this.direction_input_b);
		
		this.driection_form.appendChild(input_wrap_div_b);
		
		// Undo button
		this.undo_directions_button = document.createElement('input');
		this.undo_directions_button.setAttribute('type', 'button');
		this.undo_directions_button.setAttribute('disabled', true);
		this.undo_directions_button.className = 'kd-button kd-button-submit';
		this.undo_directions_button.value = L10n.undo;
		this.undo_directions_button.onclick = function(event){
			instance.cancel_default_action(event);
			instance.undo_directions();
			return false;
		};
		
		// Clear button
		this.clear_directions_button = document.createElement('input');
		this.clear_directions_button.setAttribute('type', 'button');
		this.clear_directions_button.className = 'kd-button kd-button-submit';
		this.clear_directions_button.value = L10n.clear;
		this.clear_directions_button.onclick = function(event){
			instance.cancel_default_action(event);
			instance.clear_directions();
			return false;
		};
		
		// Reverse button
		this.reverse_directions_button = document.createElement('div');
		this.reverse_directions_button.className = 'reverse-directions';
		this.reverse_directions_button.title = L10n.reverse_directions;
		this.reverse_directions_button.onclick = function(event){
			instance.cancel_default_action(event);
			instance.reverse_directions();
			return false;
		};
		
		// Submit button
		var submit_directions = document.createElement('input');
		submit_directions.setAttribute('type', 'submit');
		submit_directions.className = 'kd-button kd-button-submit';
		submit_directions.value = L10n.get_directions;
		
		
		var input_wrap_div_submit = document.createElement('div');
		input_wrap_div_submit.className = 'dir-input';
		input_wrap_div_submit.appendChild(this.reverse_directions_button);
		input_wrap_div_submit.appendChild(submit_directions);
		input_wrap_div_submit.appendChild(this.undo_directions_button);
		input_wrap_div_submit.appendChild(this.clear_directions_button);
		
		this.driection_form.appendChild(input_wrap_div_submit);
		
		google.maps.event.addDomListener(this.directions_grabber, 'click', function(){
			instance.toggle_directions_container();
		});
		
		this.directions_response = document.createElement('div');
		this.directions_response.className = 'map-directions-response';
		this.directions_response.style.width = '300px';
		this.directions_response.style.display = 'none';
		this.directions_response.style.padding = '0px';
		this.directions_response.style.margin = '0px';
		
		this.directions_div.appendChild(this.directions_response);		
		this.directions_div.index = 100000000;
		
		this.map.controls[google.maps.ControlPosition.TOP_LEFT].push(this.directions_div);
	};
	
	ResponsiveGoogleMap.prototype.cancel_default_action = function(e){
		var evt = e ? e:window.event;
		if (evt.preventDefault)
			evt.preventDefault();
		evt.returnValue = false;
		return false;
	};
	
	ResponsiveGoogleMap.prototype.toggle_directions_container = function(direction){
		if(direction == 'close')
			close_directions_container(this);
		else if(direction == 'open')
			open_directions_container(this);
		else if(this.directions_div_status !== 'opened')
			open_directions_container(this);
		else
			close_directions_container(this);
	};
	
	ResponsiveGoogleMap.prototype.get_infowindow_content = function(location){
		var instance = this;
		var directions_address = location.geocode_address;
		if(!directions_address)
			directions_address = location.lat + ', ' + location.lng;

		var content_string = document.createElement('div');
		content_string.className = 'map_style_container infowindow-content';
		content_string.innerHTML = location.infowindow_content;
		
		if(Boolean(parseInt(this.getSetting('enable_directions'))) && Boolean(parseInt(this.getSetting('enable_infowindow_directions')))){
			var marker_directions = document.createElement('div');
			marker_directions.className = 'marker_directions';
			
			var strong = document.createElement('strong');
			strong.innerHTML = L10n.directions+'&nbsp;&nbsp;';
			marker_directions.appendChild(strong);
			
			var dir_from = document.createElement('a');
			dir_from.setAttribute('href', '#');
			dir_from.title = directions_address;
			dir_from.innerHTML = L10n.from_here;
			dir_from.onclick = function(event){
				instance.cancel_default_action(event);
				instance.toggle_directions_container('open');
				instance.direction_input_a.value = directions_address;
				instance.direction_input_b.focus();
				return false;
			};
			marker_directions.appendChild(dir_from);
			
			var span = document.createElement('span');
			span.innerHTML = '&nbsp;&nbsp;';
			marker_directions.appendChild(span);
			
			var dir_to = document.createElement('a');
			dir_to.setAttribute('href', '#');
			dir_to.title = directions_address;
			dir_to.innerHTML = L10n.to_here;
			dir_to.onclick = function(event){
				instance.cancel_default_action(event);
				instance.toggle_directions_container('open');
				instance.direction_input_b.value = directions_address;
				instance.direction_input_a.focus();
				return false;
			};
			marker_directions.appendChild(dir_to);
			content_string.appendChild(marker_directions);
		}
		
		if(Boolean(parseInt(this.getSetting('enable_infowindow_zoom_control')))){
			var zoom_controls = document.createElement('div');
			zoom_controls.className = 'zoom_controls';
			
			var zoom_strong = document.createElement('strong');
			zoom_strong.innerHTML = L10n.zoom+'&nbsp;&nbsp;';
			zoom_controls.appendChild(zoom_strong);
			
			var zoom_in = document.createElement('a');
			zoom_in.setAttribute('href', '#');
			zoom_in.title = L10n.zoom_in;
			zoom_in.innerHTML = L10n.intext;
			zoom_in.onclick = function(event){
				instance.cancel_default_action(event);
				var zoom = instance.map.getZoom();
				zoom = zoom + 2;
				instance.close_all_infowindows();
				instance.map.setZoom(zoom);
				var marker = instance.get_marker_by_id(location.id);
				var infowindow = instance.get_infowindow_by_id(location.id);
				instance.map.setCenter(marker.position);
				infowindow.open(instance.map,marker);
				return false;
			};
			zoom_controls.appendChild(zoom_in);
			
			var zoom_span = document.createElement('span');
			zoom_span.innerHTML = '&nbsp;&nbsp;';
			zoom_controls.appendChild(zoom_span);
			
			var zoom_out = document.createElement('a');
			zoom_out.setAttribute('href', '#');
			zoom_out.title = L10n.zoom_out;
			zoom_out.innerHTML = L10n.out;
			zoom_out.onclick = function(event){
				instance.cancel_default_action(event);
				var zoom = instance.map.getZoom();
				zoom = zoom - 2;
				instance.close_all_infowindows();
				instance.map.setZoom(zoom);
				var marker = instance.get_marker_by_id(location.id);
				var infowindow = instance.get_infowindow_by_id(location.id);
				instance.map.setCenter(marker.position);
				infowindow.open(instance.map,marker);
				return false;
			};
			zoom_controls.appendChild(zoom_out);
			
			var zoom_span2 = document.createElement('span');
			zoom_span2.innerHTML = '&nbsp;&nbsp;';
			zoom_controls.appendChild(zoom_span2);
			
			var zoom_reset = document.createElement('a');
			zoom_reset.setAttribute('href', '#');
			zoom_reset.title = L10n.reset_zoom;
			zoom_reset.innerHTML = L10n.resettext;
			zoom_reset.onclick = function(event){
				instance.cancel_default_action(event);
				var zoom = instance.map.getZoom();
				zoom = zoom - 2;
				instance.close_all_infowindows();
				instance.map.setZoom(instance.map_options.zoom);
				var marker = instance.get_marker_by_id(location.id);
				var infowindow = instance.get_infowindow_by_id(location.id);
				instance.map.setCenter(marker.position);
				infowindow.open(instance.map,marker);
				return false;
			};
			zoom_controls.appendChild(zoom_reset);
			content_string.appendChild(zoom_controls);
		}
		return content_string;
	};
	
	// Closes all infowindows on the map
	ResponsiveGoogleMap.prototype.close_all_infowindows = function(){
		for(var i=0;i<this.infowindow_array.length;i++)
			this.infowindow_array[i].close(this.map,this.marker_array[i]);
	};
	
	// calculate to directions route
	ResponsiveGoogleMap.prototype.calc_route = function(){
		this.close_all_infowindows();
		var instance = this;
		// First, remove any existing markers from the map.
		for(i=0;i<this.directions_array.length;i++){
			this.directions_array[i].setMap(null);
		}
		// Now, clear the array itself.
		this.directions_array = [];
		
		var start = this.direction_input_a.value;
		if((!start || start.toLowerCase() == L10n.my_location) && instance.user_location != null)
			start = instance.user_location.geocode_address;
		var end = this.direction_input_b.value;
		if((!end || end.toLowerCase() == L10n.my_location) && instance.user_location != null)
			end = instance.user_location.geocode_address;
		
		var request = {
			origin:start,
			destination:end,
			travelMode: google.maps.DirectionsTravelMode.DRIVING
		};
		this.directions_service.route(request, function(response, status){
			if (status == google.maps.DirectionsStatus.OK){
				instance.directions_display.setDirections(response);
				instance.show_steps(response);
			}
		});
	};
	
	// Show the map steps	
	ResponsiveGoogleMap.prototype.show_steps = function(direction_result){
		if(direction_result === null) return;
		// For each step, place a marker, and add the text to the marker's
		// info window. Also attach the marker to an array so we
		// can keep track of it and remove it when calculating new
		// routes.
		for(i=0;i<this.directions_array.length;i++){
			this.directions_array[i].setMap(null);
		}
		// Now, clear the array itself.
		this.directions_array = [];
		var route = direction_result.routes[0].legs[0];
		//reset the bounds
		this.bounds = new google.maps.LatLngBounds();
		for(i=0;i<this.marker_array.length;i++){
			this.bounds.extend(this.marker_array[i].position);
		}
		
		for(var i=0;i< route.steps.length;i++){
			var marker = new google.maps.Marker({
				position: route.steps[i].start_point, 
				map: this.map
			});
			//add directions to the bounds
			this.bounds.extend(marker.position);	
			this.attach_instruction_text(marker, route.steps[i].instructions);
			this.directions_array[i] = marker;
		}
	};
	
	// Attach instruction to a single step
	ResponsiveGoogleMap.prototype.attach_instruction_text = function(marker, text){
		var instance = this;
		google.maps.event.addListener(marker, 'click', function(){
			// Open an info window when the marker is clicked on,
			// containing the text of the step.
			instance.step_display.setContent(text);
			instance.step_display.open(instance.map, marker);
		});
	};
	
	// undo the disabling of the undo direction button
	ResponsiveGoogleMap.prototype.set_undo_disabled = function(value){
		if(value)
			this.undo_directions_button.disabled = true;
		else
			this.undo_directions_button.disabled = false;
	};
	
	ResponsiveGoogleMap.prototype.reverse_directions = function(){
		var instance = this;
		var way_points = new Array();
		for(var i=0;i<this.current_directions.routes[0].legs[0].via_waypoint.length;i++){
			way_points.push({
				location: this.current_directions.routes[0].legs[0].via_waypoint[i].location,
				stopover: false
			});
		}
		way_points.reverse();
		var request = {
			origin:this.current_directions.routes[0].legs[0].end_address,
			destination:this.current_directions.routes[0].legs[0].start_address,
			waypoints:way_points,
			travelMode: google.maps.DirectionsTravelMode.DRIVING
		};
		this.directions_service.route(request, function(response, status){
			if (status == google.maps.DirectionsStatus.OK){
				instance.directions_display.setDirections(response);
				instance.show_steps(response);
			}
		});
	};
	
	ResponsiveGoogleMap.prototype.set_reverse_directions = function(){
		if(this.current_directions || this.old_directions.length){
			this.reverse_directions_button.style.display = 'block';
			this.clear_directions_button.disabled = false;
		}
		else {
			this.reverse_directions_button.style.display = 'none';
			this.clear_directions_button.disabled = true;
		}
	};

	//Undo the direction to the last request
	ResponsiveGoogleMap.prototype.undo_directions = function()	{
		this.current_directions = null;
		this.directions_display.setDirections(this.old_directions.pop());
		if(!this.old_directions.length)
			this.set_undo_disabled(true);
	};
	
	// close lage view / goto small view
	ResponsiveGoogleMap.prototype.goto_small_map = function(){
		var center = this.map.getCenter();
		this._fullscreen = false;
		this.close_text.style.display = 'none';
		if(this.fullscreen_text){
			this.fullscreen_text.style.display = 'block';
			this.exit_fullscreen_text.style.display = 'none';
		}
		this.fullscreen_icon.style.display = 'block';
		$('html').height('auto');
		$(this.map_container).removeClass('large_map_container').hide().addClass('small_map_container').fadeIn(parseInt(this.getSetting('fade_speed')));
		google.maps.event.trigger(this.map, "resize");
		this.map.setCenter(center);
		this.directions_display.setPanel(this.directions_response_div);
		$(this.directions_response).hide();
		$('html').css('overflow', 'scroll');
		window.scrollTo(this.scroll_pos.x,this.scroll_pos.y);
		//if(!this.is_basic && this.current_directions)
			//this.fit_bounds();
		return false;
	};
	
	// open large view
	ResponsiveGoogleMap.prototype.goto_large_map = function(){
		var center = this.map.getCenter();
		this._fullscreen = true;
		this.close_text.style.display = 'block';
		if(this.fullscreen_text){
			this.fullscreen_text.style.display = 'none';
			this.exit_fullscreen_text.style.display = 'block';
		}
		this.fullscreen_icon.style.display = 'none';
		//get current scroll position to set back later
		this.scroll_pos = {x:window.scrollX,y:window.scrollY};
		window.scrollTo(0,0);
		$('html').height($(window).outerHeight());
		$('html').css('overflow', 'hidden');
		$(this.map_container).removeClass('small_map_container').hide().addClass('large_map_container').fadeIn(parseInt(this.getSetting('fade_speed')));
		google.maps.event.trigger(this.map, "resize");
		this.map.setCenter(center);
		set_direction_max_height(this);
		this.directions_display.setPanel(this.directions_response);
		$(this.directions_response).show();
		//if(!this.is_basic && this.current_directions)
			//this.fit_bounds();
		return false;
	};
	
	ResponsiveGoogleMap.prototype.infowindow_reset = function(id){
		this.close_all_infowindows();
		this.map.setZoom(this.map_options.zoom);
		this.map.setCenter(this.marker_array[id].position);
		this.infowindow_array[id].open(this.map,this.marker_array[id]);
	};
	
	ResponsiveGoogleMap.prototype.responsive_movement = function(){
		if(this.is_basic)
			return;
		var w = this.map.getDiv().offsetWidth;
		var h = this.map.getDiv().offsetHeight;
		if(Boolean(parseInt(this.getSetting('enable_fullscreen_view')))){
			if(window.innerWidth<=400){
				if(this.fullscreen_text)
					this.fullscreen_text.style.display = 'none';
				this.fullscreen_icon.style.display = 'none';
			}
			else if(!this._fullscreen){
				if(this.fullscreen_text)
					this.fullscreen_text.style.display = 'block';
				this.fullscreen_icon.style.display = 'block';
			}
		}
		
		if(!Boolean(parseInt(this.getSetting('enable_map_options_dropdown')))){
			var show_dropdown = false;
			if(Boolean(parseInt(this.getSetting('enable_autofit_button'))) && 
				parseInt(this.center_text.style.right) + $(this.center_text).outerWidth() + 75 > w){
				this.center_text.style.display = 'none';
				show_dropdown = true;
			}
			if(show_dropdown || 
				(Boolean(parseInt(this.getSetting('enable_map_reset_button'))) && 
				parseInt(this.reset_text.style.right) + $(this.reset_text).outerWidth() + 75 > w)){
				this.reset_text.style.display = 'none';
				show_dropdown = true;
			}
			
			if(show_dropdown)
				this.map_button_dropdown.style.display = 'block';
			else {
				this.map_button_dropdown.style.display = 'none';
				if(Boolean(parseInt(this.getSetting('enable_autofit_button'))))
					this.center_text.style.display = 'block';
				if(Boolean(parseInt(this.getSetting('enable_map_reset_button'))))
					this.reset_text.style.display = 'block';
			}
		}
		
		if(parseInt(this.map_button_dropdown.style.right) + $(this.map_button_dropdown).outerWidth() + 40 > w)
			this.map_button_dropdown.style.display = 'none';
		else if(Boolean(parseInt(this.getSetting('enable_map_options_dropdown'))))
			this.map_button_dropdown.style.display = 'block';
		
		if(Boolean(parseInt(this.getSetting('enable_directions')))){
			if(w<=300 && this.directions_div){
				if(Boolean(parseInt(this.getSetting('enable_map_options_dropdown'))))
					this.get_directions_button.style.display = 'none';
				this.directions_div.style.display = 'none';
			}
			else if(this.directions_div){
				if(Boolean(parseInt(this.getSetting('enable_map_options_dropdown'))))
					this.get_directions_button.style.display = 'block';
				this.directions_div.style.display = 'block';
			}
		}

		if(h<250 || w<=400){
			this.map.setOptions({overviewMapControl:false});
			this.map.setOptions({panControl:false});
		}
		else {
			this.map.setOptions({overviewMapControl:Boolean(parseInt(this.getSetting('show_overview_map_control')))});
			this.map.setOptions({panControl:Boolean(parseInt(this.getSetting('enable_pan_control')))});
		}
	};
	// Static Methods
	ResponsiveGoogleMap.generate_uid = function(separator){
		var delim = separator || "-";
		function S4(){
			return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
		}
		return (S4() + S4() + delim + S4() + delim + S4() + delim + S4() + delim + S4() + S4() + S4());
	};
	ResponsiveGoogleMap.update_position = function(marker){
		var position = marker.getPosition();
		$('.geocoded_address_lat_single').val(position.lat());
		$('.geocoded_address_lng_single').val(position.lng());
	};
	// Event bindings
	ResponsiveGoogleMap.prototype.bind_events = function(){
		var instance = this;
		google.maps.event.addListener(instance.directions_display, 'directions_changed',function(){
			if(instance.current_directions){
				instance.old_directions.push(instance.current_directions);
				instance.set_undo_disabled(false);
			}
			instance.current_directions = instance.directions_display.getDirections();
			instance.show_steps(instance.current_directions);
			instance.set_reverse_directions();
		});
		google.maps.event.addListenerOnce(this.map, 'tilesloaded', function(){
			setTimeout(function(){
				instance.adjust_map_dropdown();
			},250);
			instance.responsive_movement();
			instance.resopnsive_timer = null;
			google.maps.event.addListener(instance.map, 'bounds_changed', function(){
				if(instance.resopnsive_timer)
					window.clearTimeout(instance.resopnsive_timer);
				instance.resopnsive_timer = setTimeout(function(){
					instance.responsive_movement();
				},250);
			});
			// Fix the IE copyright problem
			$(instance.map_container+' .gmnoprint').find('span:contains("©")').parents('.gmnoprint').addClass('ie-copyright-fix');
		});
		$(window).on('resize', '', instance, function(event){
			set_direction_max_height(instance);
			keep_map_center(instance);
		});
		$(window).on('keyup', '', instance, function(event){
			if(event.keyCode == 27 && event.data._fullscreen == true)
				event.data.goto_small_map();
		});
		$(document).on('click', '#'+instance.map_canvas.id+'_directions_response .adp-directions', function(event){
			event.preventDefault();
			$('html,body').animate({	scrollTop: $(instance.map_canvas).offset().top	}, 500);
			return false;
		});
	};
	// Private Methods
	function open_directions_container(instance){
		$(instance.directions_div).animate({width: '300px'}, 350);
		instance.directions_div_status = 'opened';
		instance.directions_grabber.innerHTML = "<span>&gt;</span>";
	};
	function close_directions_container(instance){
		$(instance.directions_div).animate({width: '0px'}, 350);
		instance.directions_div_status = 'closed';
		instance.directions_grabber.innerHTML = "<span>&lt;</span>";
	};
	function hex_to_hrgb(hex){
		hex = typeof hex !== 'undefined' ? hex : '#000000';
		// Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
		var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
		hex = hex.replace(shorthandRegex, function(m, r, g, b){
			return r + r + g + g + b + b;
		});
		var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
		return result ? {
			h: hex.substr(1),
			r: parseInt(result[1], 16),
			g: parseInt(result[2], 16),
			b: parseInt(result[3], 16)
		} : null;
	};
	function keep_map_center(instance){
		if(instance._resize_timer === false){
			instance._resize_timer = true;
			var center = instance.map.getCenter();
			setTimeout(function(){
				instance.map.setCenter(center);
				instance._resize_timer = false;
			}, 250);
		}
	};
	function set_direction_max_height(instance){
		var directions_size = $(instance.directions_ui).height() + 150; 
		$(instance.directions_response).css('max-height', $(window).height() - directions_size);
	};
	function setup_directions(instance)
	{
		instance.directions_service = new google.maps.DirectionsService();
		if(instance.directions_display != null){
			instance.directions_display.setMap(null);
    		instance.directions_display = null;
		}
		instance.directions_display = new google.maps.DirectionsRenderer({
			map: instance.map,
			draggable: true,
			suppressMarkers : true
		});
		instance.directions_response_div =  document.getElementById(instance.map_canvas.id+"_directions_response");
		if(typeof(instance.directions_response_div) != 'undefined' && instance.directions_response_div != null){
			instance.directions_response_div.innerHTML = '';
			instance.directions_display.setPanel(instance.directions_response_div);
		}
		else {
			instance.directions_response_div = document.createElement('div');
			instance.directions_response_div.id = instance.map_canvas.id+"_directions_response";
			instance.directions_response_div.className = "directions-response";
			var container = document.getElementById(instance.map_canvas.id+'_container');
			if(typeof(container) != 'undefined' && container != null){
				container.parentNode.insertBefore(instance.directions_response_div, container.nextSibling);
				instance.directions_display.setPanel(instance.directions_response_div);
			}
		}
		instance.step_display = new google.maps.InfoWindow();
		instance.set_undo_disabled(true);
	};
	
	return ResponsiveGoogleMap;
})(jQuery = typeof jQuery !== 'undefined' ? jQuery : null,
	global_map_settings = typeof global_map_settings !== 'undefined' ? global_map_settings : null,
	available_map_styles = typeof available_map_styles !== 'undefined' ? available_map_styles : null,
	RGMLocal = typeof RGMLocal !== 'undefined' ? RGMLocal : null);